# coding: utf-8

"""
    Demisto API

    This is the public REST API to integrate with the demisto server. HTTP request can be sent using any HTTP-client.  For an example dedicated client take a look at: https://github.com/demisto/demisto-py.  Requests must include API-key that can be generated in the Demisto web client under 'Settings' -> 'Integrations' -> 'API keys'   Optimistic Locking and Versioning\\:  When using Demisto REST API, you will need to make sure to work on the latest version of the item (incident, entry, etc.), otherwise, you will get a DB version error (which not allow you to override a newer item). In addition, you can pass 'version\\: -1' to force data override (make sure that other users data might be lost).  Assume that Alice and Bob both read the same data from Demisto server, then they both changed the data, and then both tried to write the new versions back to the server. Whose changes should be saved? Alice’s? Bob’s? To solve this, each data item in Demisto has a numeric incremental version. If Alice saved an item with version 4 and Bob trying to save the same item with version 3, Demisto will rollback Bob request and returns a DB version conflict error. Bob will need to get the latest item and work on it so Alice work will not get lost.  Example request using 'curl'\\:  ``` curl 'https://hostname:443/incidents/search' -H 'content-type: application/json' -H 'accept: application/json' -H 'Authorization: <API Key goes here>' --data-binary '{\"filter\":{\"query\":\"-status:closed -category:job\",\"period\":{\"by\":\"day\",\"fromValue\":7}}}' --compressed ```  # noqa: E501

    OpenAPI spec version: 2.0.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six


class Argument(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'auto': 'str',
        'default': 'bool',
        'default_value': 'str',
        'deprecated': 'bool',
        'description': 'str',
        'is_array': 'bool',
        'name': 'str',
        'predefined': 'list[str]',
        'required': 'bool',
        'secret': 'bool'
    }

    attribute_map = {
        'auto': 'auto',
        'default': 'default',
        'default_value': 'defaultValue',
        'deprecated': 'deprecated',
        'description': 'description',
        'is_array': 'isArray',
        'name': 'name',
        'predefined': 'predefined',
        'required': 'required',
        'secret': 'secret'
    }

    def __init__(self, auto=None, default=None, default_value=None, deprecated=None, description=None, is_array=None, name=None, predefined=None, required=None, secret=None):  # noqa: E501
        """Argument - a model defined in Swagger"""  # noqa: E501

        self._auto = None
        self._default = None
        self._default_value = None
        self._deprecated = None
        self._description = None
        self._is_array = None
        self._name = None
        self._predefined = None
        self._required = None
        self._secret = None
        self.discriminator = None

        if auto is not None:
            self.auto = auto
        if default is not None:
            self.default = default
        if default_value is not None:
            self.default_value = default_value
        if deprecated is not None:
            self.deprecated = deprecated
        if description is not None:
            self.description = description
        if is_array is not None:
            self.is_array = is_array
        if name is not None:
            self.name = name
        if predefined is not None:
            self.predefined = predefined
        if required is not None:
            self.required = required
        if secret is not None:
            self.secret = secret

    @property
    def auto(self):
        """Gets the auto of this Argument.  # noqa: E501


        :return: The auto of this Argument.  # noqa: E501
        :rtype: str
        """
        return self._auto

    @auto.setter
    def auto(self, auto):
        """Sets the auto of this Argument.


        :param auto: The auto of this Argument.  # noqa: E501
        :type: str
        """

        self._auto = auto

    @property
    def default(self):
        """Gets the default of this Argument.  # noqa: E501


        :return: The default of this Argument.  # noqa: E501
        :rtype: bool
        """
        return self._default

    @default.setter
    def default(self, default):
        """Sets the default of this Argument.


        :param default: The default of this Argument.  # noqa: E501
        :type: bool
        """

        self._default = default

    @property
    def default_value(self):
        """Gets the default_value of this Argument.  # noqa: E501


        :return: The default_value of this Argument.  # noqa: E501
        :rtype: str
        """
        return self._default_value

    @default_value.setter
    def default_value(self, default_value):
        """Sets the default_value of this Argument.


        :param default_value: The default_value of this Argument.  # noqa: E501
        :type: str
        """

        self._default_value = default_value

    @property
    def deprecated(self):
        """Gets the deprecated of this Argument.  # noqa: E501


        :return: The deprecated of this Argument.  # noqa: E501
        :rtype: bool
        """
        return self._deprecated

    @deprecated.setter
    def deprecated(self, deprecated):
        """Sets the deprecated of this Argument.


        :param deprecated: The deprecated of this Argument.  # noqa: E501
        :type: bool
        """

        self._deprecated = deprecated

    @property
    def description(self):
        """Gets the description of this Argument.  # noqa: E501


        :return: The description of this Argument.  # noqa: E501
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """Sets the description of this Argument.


        :param description: The description of this Argument.  # noqa: E501
        :type: str
        """

        self._description = description

    @property
    def is_array(self):
        """Gets the is_array of this Argument.  # noqa: E501


        :return: The is_array of this Argument.  # noqa: E501
        :rtype: bool
        """
        return self._is_array

    @is_array.setter
    def is_array(self, is_array):
        """Sets the is_array of this Argument.


        :param is_array: The is_array of this Argument.  # noqa: E501
        :type: bool
        """

        self._is_array = is_array

    @property
    def name(self):
        """Gets the name of this Argument.  # noqa: E501


        :return: The name of this Argument.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this Argument.


        :param name: The name of this Argument.  # noqa: E501
        :type: str
        """

        self._name = name

    @property
    def predefined(self):
        """Gets the predefined of this Argument.  # noqa: E501


        :return: The predefined of this Argument.  # noqa: E501
        :rtype: list[str]
        """
        return self._predefined

    @predefined.setter
    def predefined(self, predefined):
        """Sets the predefined of this Argument.


        :param predefined: The predefined of this Argument.  # noqa: E501
        :type: list[str]
        """

        self._predefined = predefined

    @property
    def required(self):
        """Gets the required of this Argument.  # noqa: E501


        :return: The required of this Argument.  # noqa: E501
        :rtype: bool
        """
        return self._required

    @required.setter
    def required(self, required):
        """Sets the required of this Argument.


        :param required: The required of this Argument.  # noqa: E501
        :type: bool
        """

        self._required = required

    @property
    def secret(self):
        """Gets the secret of this Argument.  # noqa: E501


        :return: The secret of this Argument.  # noqa: E501
        :rtype: bool
        """
        return self._secret

    @secret.setter
    def secret(self, secret):
        """Sets the secret of this Argument.


        :param secret: The secret of this Argument.  # noqa: E501
        :type: bool
        """

        self._secret = secret

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(Argument, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, Argument):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



from __future__ import print_function
import demisto_client.demisto_api
from demisto_client.demisto_api.rest import ApiException
from pprint import pprint

api_key = None  # set to your 'YOUR_API_KEY' or set environment variable: DEMISTO_API_KEY
base_url = None  # set to your 'http://DEMISTO_HOST' or set environment variable: DEMISTO_BASE_URL

# create an instance of the API class
api_instance = demisto_client.configure(base_url=base_url, api_key=api_key, debug=True)
filter = demisto_client.demisto_api.SearchIncidentsData()

# Create incident filter object
inc_filter = demisto_client.demisto_api.IncidentFilter()
inc_filter.name = ['test']

filter.filter = inc_filter

try:
    # Search incidents by filter
    api_response = api_instance.search_incidents(filter=filter)
    pprint(api_response)
except ApiException as e:
    print("Exception when calling DefaultApi->search_incidents: %s\n" % e)



# coding: utf-8

"""
    Demisto API

    This is the public REST API to integrate with the demisto server. HTTP request can be sent using any HTTP-client.  For an example dedicated client take a look at: https://github.com/demisto/demisto-py.  Requests must include API-key that can be generated in the Demisto web client under 'Settings' -> 'Integrations' -> 'API keys'   Optimistic Locking and Versioning\\:  When using Demisto REST API, you will need to make sure to work on the latest version of the item (incident, entry, etc.), otherwise, you will get a DB version error (which not allow you to override a newer item). In addition, you can pass 'version\\: -1' to force data override (make sure that other users data might be lost).  Assume that Alice and Bob both read the same data from Demisto server, then they both changed the data, and then both tried to write the new versions back to the server. Whose changes should be saved? Alice’s? Bob’s? To solve this, each data item in Demisto has a numeric incremental version. If Alice saved an item with version 4 and Bob trying to save the same item with version 3, Demisto will rollback Bob request and returns a DB version conflict error. Bob will need to get the latest item and work on it so Alice work will not get lost.  Example request using 'curl'\\:  ``` curl 'https://hostname:443/incidents/search' -H 'content-type: application/json' -H 'accept: application/json' -H 'Authorization: <API Key goes here>' --data-binary '{\"filter\":{\"query\":\"-status:closed -category:job\",\"period\":{\"by\":\"day\",\"fromValue\":7}}}' --compressed ```  # noqa: E501

    OpenAPI spec version: 2.0.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six

from demisto_client.demisto_api.models.argument import Argument  # noqa: F401,E501
from demisto_client.demisto_api.models.output import Output  # noqa: F401,E501
from demisto_client.demisto_api.models.script_target import ScriptTarget  # noqa: F401,E501
from demisto_client.demisto_api.models.script_type import ScriptType  # noqa: F401,E501


class ScriptAPI(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'arguments': 'list[Argument]',
        'comment': 'str',
        'context_keys': 'list[str]',
        'depends_on': 'dict(str, list[str])',
        'deprecated': 'bool',
        'detached': 'bool',
        'docker_image': 'str',
        'enabled': 'bool',
        'hidden': 'bool',
        'id': 'str',
        'locked': 'bool',
        'modified': 'datetime',
        'name': 'str',
        'outputs': 'list[Output]',
        'permitted': 'bool',
        'propagation_labels': 'list[str]',
        'roles': 'list[str]',
        'run_as': 'str',
        'script_target': 'ScriptTarget',
        'system': 'bool',
        'tags': 'list[str]',
        'type': 'ScriptType',
        'user': 'str',
        'version': 'int'
    }

    attribute_map = {
        'arguments': 'arguments',
        'comment': 'comment',
        'context_keys': 'contextKeys',
        'depends_on': 'dependsOn',
        'deprecated': 'deprecated',
        'detached': 'detached',
        'docker_image': 'dockerImage',
        'enabled': 'enabled',
        'hidden': 'hidden',
        'id': 'id',
        'locked': 'locked',
        'modified': 'modified',
        'name': 'name',
        'outputs': 'outputs',
        'permitted': 'permitted',
        'propagation_labels': 'propagationLabels',
        'roles': 'roles',
        'run_as': 'runAs',
        'script_target': 'scriptTarget',
        'system': 'system',
        'tags': 'tags',
        'type': 'type',
        'user': 'user',
        'version': 'version'
    }

    def __init__(self, arguments=None, comment=None, context_keys=None, depends_on=None, deprecated=None, detached=None, docker_image=None, enabled=None, hidden=None, id=None, locked=None, modified=None, name=None, outputs=None, permitted=None, propagation_labels=None, roles=None, run_as=None, script_target=None, system=None, tags=None, type=None, user=None, version=None):  # noqa: E501
        """ScriptAPI - a model defined in Swagger"""  # noqa: E501

        self._arguments = None
        self._comment = None
        self._context_keys = None
        self._depends_on = None
        self._deprecated = None
        self._detached = None
        self._docker_image = None
        self._enabled = None
        self._hidden = None
        self._id = None
        self._locked = None
        self._modified = None
        self._name = None
        self._outputs = None
        self._permitted = None
        self._propagation_labels = None
        self._roles = None
        self._run_as = None
        self._script_target = None
        self._system = None
        self._tags = None
        self._type = None
        self._user = None
        self._version = None
        self.discriminator = None

        if arguments is not None:
            self.arguments = arguments
        if comment is not None:
            self.comment = comment
        if context_keys is not None:
            self.context_keys = context_keys
        if depends_on is not None:
            self.depends_on = depends_on
        if deprecated is not None:
            self.deprecated = deprecated
        if detached is not None:
            self.detached = detached
        if docker_image is not None:
            self.docker_image = docker_image
        if enabled is not None:
            self.enabled = enabled
        if hidden is not None:
            self.hidden = hidden
        if id is not None:
            self.id = id
        if locked is not None:
            self.locked = locked
        if modified is not None:
            self.modified = modified
        if name is not None:
            self.name = name
        if outputs is not None:
            self.outputs = outputs
        if permitted is not None:
            self.permitted = permitted
        if propagation_labels is not None:
            self.propagation_labels = propagation_labels
        if roles is not None:
            self.roles = roles
        if run_as is not None:
            self.run_as = run_as
        if script_target is not None:
            self.script_target = script_target
        if system is not None:
            self.system = system
        if tags is not None:
            self.tags = tags
        if type is not None:
            self.type = type
        if user is not None:
            self.user = user
        if version is not None:
            self.version = version

    @property
    def arguments(self):
        """Gets the arguments of this ScriptAPI.  # noqa: E501


        :return: The arguments of this ScriptAPI.  # noqa: E501
        :rtype: list[Argument]
        """
        return self._arguments

    @arguments.setter
    def arguments(self, arguments):
        """Sets the arguments of this ScriptAPI.


        :param arguments: The arguments of this ScriptAPI.  # noqa: E501
        :type: list[Argument]
        """

        self._arguments = arguments

    @property
    def comment(self):
        """Gets the comment of this ScriptAPI.  # noqa: E501


        :return: The comment of this ScriptAPI.  # noqa: E501
        :rtype: str
        """
        return self._comment

    @comment.setter
    def comment(self, comment):
        """Sets the comment of this ScriptAPI.


        :param comment: The comment of this ScriptAPI.  # noqa: E501
        :type: str
        """

        self._comment = comment

    @property
    def context_keys(self):
        """Gets the context_keys of this ScriptAPI.  # noqa: E501


        :return: The context_keys of this ScriptAPI.  # noqa: E501
        :rtype: list[str]
        """
        return self._context_keys

    @context_keys.setter
    def context_keys(self, context_keys):
        """Sets the context_keys of this ScriptAPI.


        :param context_keys: The context_keys of this ScriptAPI.  # noqa: E501
        :type: list[str]
        """

        self._context_keys = context_keys

    @property
    def depends_on(self):
        """Gets the depends_on of this ScriptAPI.  # noqa: E501


        :return: The depends_on of this ScriptAPI.  # noqa: E501
        :rtype: dict(str, list[str])
        """
        return self._depends_on

    @depends_on.setter
    def depends_on(self, depends_on):
        """Sets the depends_on of this ScriptAPI.


        :param depends_on: The depends_on of this ScriptAPI.  # noqa: E501
        :type: dict(str, list[str])
        """

        self._depends_on = depends_on

    @property
    def deprecated(self):
        """Gets the deprecated of this ScriptAPI.  # noqa: E501


        :return: The deprecated of this ScriptAPI.  # noqa: E501
        :rtype: bool
        """
        return self._deprecated

    @deprecated.setter
    def deprecated(self, deprecated):
        """Sets the deprecated of this ScriptAPI.


        :param deprecated: The deprecated of this ScriptAPI.  # noqa: E501
        :type: bool
        """

        self._deprecated = deprecated

    @property
    def detached(self):
        """Gets the detached of this ScriptAPI.  # noqa: E501


        :return: The detached of this ScriptAPI.  # noqa: E501
        :rtype: bool
        """
        return self._detached

    @detached.setter
    def detached(self, detached):
        """Sets the detached of this ScriptAPI.


        :param detached: The detached of this ScriptAPI.  # noqa: E501
        :type: bool
        """

        self._detached = detached

    @property
    def docker_image(self):
        """Gets the docker_image of this ScriptAPI.  # noqa: E501


        :return: The docker_image of this ScriptAPI.  # noqa: E501
        :rtype: str
        """
        return self._docker_image

    @docker_image.setter
    def docker_image(self, docker_image):
        """Sets the docker_image of this ScriptAPI.


        :param docker_image: The docker_image of this ScriptAPI.  # noqa: E501
        :type: str
        """

        self._docker_image = docker_image

    @property
    def enabled(self):
        """Gets the enabled of this ScriptAPI.  # noqa: E501


        :return: The enabled of this ScriptAPI.  # noqa: E501
        :rtype: bool
        """
        return self._enabled

    @enabled.setter
    def enabled(self, enabled):
        """Sets the enabled of this ScriptAPI.


        :param enabled: The enabled of this ScriptAPI.  # noqa: E501
        :type: bool
        """

        self._enabled = enabled

    @property
    def hidden(self):
        """Gets the hidden of this ScriptAPI.  # noqa: E501


        :return: The hidden of this ScriptAPI.  # noqa: E501
        :rtype: bool
        """
        return self._hidden

    @hidden.setter
    def hidden(self, hidden):
        """Sets the hidden of this ScriptAPI.


        :param hidden: The hidden of this ScriptAPI.  # noqa: E501
        :type: bool
        """

        self._hidden = hidden

    @property
    def id(self):
        """Gets the id of this ScriptAPI.  # noqa: E501


        :return: The id of this ScriptAPI.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this ScriptAPI.


        :param id: The id of this ScriptAPI.  # noqa: E501
        :type: str
        """

        self._id = id

    @property
    def locked(self):
        """Gets the locked of this ScriptAPI.  # noqa: E501


        :return: The locked of this ScriptAPI.  # noqa: E501
        :rtype: bool
        """
        return self._locked

    @locked.setter
    def locked(self, locked):
        """Sets the locked of this ScriptAPI.


        :param locked: The locked of this ScriptAPI.  # noqa: E501
        :type: bool
        """

        self._locked = locked

    @property
    def modified(self):
        """Gets the modified of this ScriptAPI.  # noqa: E501


        :return: The modified of this ScriptAPI.  # noqa: E501
        :rtype: datetime
        """
        return self._modified

    @modified.setter
    def modified(self, modified):
        """Sets the modified of this ScriptAPI.


        :param modified: The modified of this ScriptAPI.  # noqa: E501
        :type: datetime
        """

        self._modified = modified

    @property
    def name(self):
        """Gets the name of this ScriptAPI.  # noqa: E501


        :return: The name of this ScriptAPI.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this ScriptAPI.


        :param name: The name of this ScriptAPI.  # noqa: E501
        :type: str
        """

        self._name = name

    @property
    def outputs(self):
        """Gets the outputs of this ScriptAPI.  # noqa: E501


        :return: The outputs of this ScriptAPI.  # noqa: E501
        :rtype: list[Output]
        """
        return self._outputs

    @outputs.setter
    def outputs(self, outputs):
        """Sets the outputs of this ScriptAPI.


        :param outputs: The outputs of this ScriptAPI.  # noqa: E501
        :type: list[Output]
        """

        self._outputs = outputs

    @property
    def permitted(self):
        """Gets the permitted of this ScriptAPI.  # noqa: E501


        :return: The permitted of this ScriptAPI.  # noqa: E501
        :rtype: bool
        """
        return self._permitted

    @permitted.setter
    def permitted(self, permitted):
        """Sets the permitted of this ScriptAPI.


        :param permitted: The permitted of this ScriptAPI.  # noqa: E501
        :type: bool
        """

        self._permitted = permitted

    @property
    def propagation_labels(self):
        """Gets the propagation_labels of this ScriptAPI.  # noqa: E501


        :return: The propagation_labels of this ScriptAPI.  # noqa: E501
        :rtype: list[str]
        """
        return self._propagation_labels

    @propagation_labels.setter
    def propagation_labels(self, propagation_labels):
        """Sets the propagation_labels of this ScriptAPI.


        :param propagation_labels: The propagation_labels of this ScriptAPI.  # noqa: E501
        :type: list[str]
        """

        self._propagation_labels = propagation_labels

    @property
    def roles(self):
        """Gets the roles of this ScriptAPI.  # noqa: E501


        :return: The roles of this ScriptAPI.  # noqa: E501
        :rtype: list[str]
        """
        return self._roles

    @roles.setter
    def roles(self, roles):
        """Sets the roles of this ScriptAPI.


        :param roles: The roles of this ScriptAPI.  # noqa: E501
        :type: list[str]
        """

        self._roles = roles

    @property
    def run_as(self):
        """Gets the run_as of this ScriptAPI.  # noqa: E501


        :return: The run_as of this ScriptAPI.  # noqa: E501
        :rtype: str
        """
        return self._run_as

    @run_as.setter
    def run_as(self, run_as):
        """Sets the run_as of this ScriptAPI.


        :param run_as: The run_as of this ScriptAPI.  # noqa: E501
        :type: str
        """

        self._run_as = run_as

    @property
    def script_target(self):
        """Gets the script_target of this ScriptAPI.  # noqa: E501


        :return: The script_target of this ScriptAPI.  # noqa: E501
        :rtype: ScriptTarget
        """
        return self._script_target

    @script_target.setter
    def script_target(self, script_target):
        """Sets the script_target of this ScriptAPI.


        :param script_target: The script_target of this ScriptAPI.  # noqa: E501
        :type: ScriptTarget
        """

        self._script_target = script_target

    @property
    def system(self):
        """Gets the system of this ScriptAPI.  # noqa: E501


        :return: The system of this ScriptAPI.  # noqa: E501
        :rtype: bool
        """
        return self._system

    @system.setter
    def system(self, system):
        """Sets the system of this ScriptAPI.


        :param system: The system of this ScriptAPI.  # noqa: E501
        :type: bool
        """

        self._system = system

    @property
    def tags(self):
        """Gets the tags of this ScriptAPI.  # noqa: E501


        :return: The tags of this ScriptAPI.  # noqa: E501
        :rtype: list[str]
        """
        return self._tags

    @tags.setter
    def tags(self, tags):
        """Sets the tags of this ScriptAPI.


        :param tags: The tags of this ScriptAPI.  # noqa: E501
        :type: list[str]
        """

        self._tags = tags

    @property
    def type(self):
        """Gets the type of this ScriptAPI.  # noqa: E501


        :return: The type of this ScriptAPI.  # noqa: E501
        :rtype: ScriptType
        """
        return self._type

    @type.setter
    def type(self, type):
        """Sets the type of this ScriptAPI.


        :param type: The type of this ScriptAPI.  # noqa: E501
        :type: ScriptType
        """

        self._type = type

    @property
    def user(self):
        """Gets the user of this ScriptAPI.  # noqa: E501


        :return: The user of this ScriptAPI.  # noqa: E501
        :rtype: str
        """
        return self._user

    @user.setter
    def user(self, user):
        """Sets the user of this ScriptAPI.


        :param user: The user of this ScriptAPI.  # noqa: E501
        :type: str
        """

        self._user = user

    @property
    def version(self):
        """Gets the version of this ScriptAPI.  # noqa: E501


        :return: The version of this ScriptAPI.  # noqa: E501
        :rtype: int
        """
        return self._version

    @version.setter
    def version(self, version):
        """Sets the version of this ScriptAPI.


        :param version: The version of this ScriptAPI.  # noqa: E501
        :type: int
        """

        self._version = version

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(ScriptAPI, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ScriptAPI):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Demisto API

    This is the public REST API to integrate with the demisto server. HTTP request can be sent using any HTTP-client.  For an example dedicated client take a look at: https://github.com/demisto/demisto-py.  Requests must include API-key that can be generated in the Demisto web client under 'Settings' -> 'Integrations' -> 'API keys'   Optimistic Locking and Versioning\\:  When using Demisto REST API, you will need to make sure to work on the latest version of the item (incident, entry, etc.), otherwise, you will get a DB version error (which not allow you to override a newer item). In addition, you can pass 'version\\: -1' to force data override (make sure that other users data might be lost).  Assume that Alice and Bob both read the same data from Demisto server, then they both changed the data, and then both tried to write the new versions back to the server. Whose changes should be saved? Alice’s? Bob’s? To solve this, each data item in Demisto has a numeric incremental version. If Alice saved an item with version 4 and Bob trying to save the same item with version 3, Demisto will rollback Bob request and returns a DB version conflict error. Bob will need to get the latest item and work on it so Alice work will not get lost.  Example request using 'curl'\\:  ``` curl 'https://hostname:443/incidents/search' -H 'content-type: application/json' -H 'accept: application/json' -H 'Authorization: <API Key goes here>' --data-binary '{\"filter\":{\"query\":\"-status:closed -category:job\",\"period\":{\"by\":\"day\",\"fromValue\":7}}}' --compressed ```  # noqa: E501

    OpenAPI spec version: 2.0.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six

from demisto_client.demisto_api.models.advance_arg import AdvanceArg  # noqa: F401,E501


class EvidenceData(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'custom_fields': 'dict(str, AdvanceArg)',
        'description': 'AdvanceArg',
        'occurred': 'AdvanceArg',
        'tags': 'AdvanceArg'
    }

    attribute_map = {
        'custom_fields': 'customFields',
        'description': 'description',
        'occurred': 'occurred',
        'tags': 'tags'
    }

    def __init__(self, custom_fields=None, description=None, occurred=None, tags=None):  # noqa: E501
        """EvidenceData - a model defined in Swagger"""  # noqa: E501

        self._custom_fields = None
        self._description = None
        self._occurred = None
        self._tags = None
        self.discriminator = None

        if custom_fields is not None:
            self.custom_fields = custom_fields
        if description is not None:
            self.description = description
        if occurred is not None:
            self.occurred = occurred
        if tags is not None:
            self.tags = tags

    @property
    def custom_fields(self):
        """Gets the custom_fields of this EvidenceData.  # noqa: E501

        This field must have empty json key  # noqa: E501

        :return: The custom_fields of this EvidenceData.  # noqa: E501
        :rtype: dict(str, AdvanceArg)
        """
        return self._custom_fields

    @custom_fields.setter
    def custom_fields(self, custom_fields):
        """Sets the custom_fields of this EvidenceData.

        This field must have empty json key  # noqa: E501

        :param custom_fields: The custom_fields of this EvidenceData.  # noqa: E501
        :type: dict(str, AdvanceArg)
        """

        self._custom_fields = custom_fields

    @property
    def description(self):
        """Gets the description of this EvidenceData.  # noqa: E501


        :return: The description of this EvidenceData.  # noqa: E501
        :rtype: AdvanceArg
        """
        return self._description

    @description.setter
    def description(self, description):
        """Sets the description of this EvidenceData.


        :param description: The description of this EvidenceData.  # noqa: E501
        :type: AdvanceArg
        """

        self._description = description

    @property
    def occurred(self):
        """Gets the occurred of this EvidenceData.  # noqa: E501


        :return: The occurred of this EvidenceData.  # noqa: E501
        :rtype: AdvanceArg
        """
        return self._occurred

    @occurred.setter
    def occurred(self, occurred):
        """Sets the occurred of this EvidenceData.


        :param occurred: The occurred of this EvidenceData.  # noqa: E501
        :type: AdvanceArg
        """

        self._occurred = occurred

    @property
    def tags(self):
        """Gets the tags of this EvidenceData.  # noqa: E501


        :return: The tags of this EvidenceData.  # noqa: E501
        :rtype: AdvanceArg
        """
        return self._tags

    @tags.setter
    def tags(self, tags):
        """Sets the tags of this EvidenceData.


        :param tags: The tags of this EvidenceData.  # noqa: E501
        :type: AdvanceArg
        """

        self._tags = tags

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(EvidenceData, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, EvidenceData):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Demisto API

    This is the public REST API to integrate with the demisto server. HTTP request can be sent using any HTTP-client.  For an example dedicated client take a look at: https://github.com/demisto/demisto-py.  Requests must include API-key that can be generated in the Demisto web client under 'Settings' -> 'Integrations' -> 'API keys'   Optimistic Locking and Versioning\\:  When using Demisto REST API, you will need to make sure to work on the latest version of the item (incident, entry, etc.), otherwise, you will get a DB version error (which not allow you to override a newer item). In addition, you can pass 'version\\: -1' to force data override (make sure that other users data might be lost).  Assume that Alice and Bob both read the same data from Demisto server, then they both changed the data, and then both tried to write the new versions back to the server. Whose changes should be saved? Alice’s? Bob’s? To solve this, each data item in Demisto has a numeric incremental version. If Alice saved an item with version 4 and Bob trying to save the same item with version 3, Demisto will rollback Bob request and returns a DB version conflict error. Bob will need to get the latest item and work on it so Alice work will not get lost.  Example request using 'curl'\\:  ``` curl 'https://hostname:443/incidents/search' -H 'content-type: application/json' -H 'accept: application/json' -H 'Authorization: <API Key goes here>' --data-binary '{\"filter\":{\"query\":\"-status:closed -category:job\",\"period\":{\"by\":\"day\",\"fromValue\":7}}}' --compressed ```  # noqa: E501

    OpenAPI spec version: 2.0.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six

from demisto_client.demisto_api.models.custom_fields import CustomFields  # noqa: F401,E501
from demisto_client.demisto_api.models.insight_cache import InsightCache  # noqa: F401,E501


class IocObject(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'custom_fields': 'CustomFields',
        'account': 'str',
        'calculated_time': 'datetime',
        'comment': 'str',
        'first_seen': 'datetime',
        'first_seen_entry_id': 'str',
        'id': 'str',
        'indicator_type': 'str',
        'insight_cache': 'InsightCache',
        'investigation_i_ds': 'list[str]',
        'last_reputation_run': 'datetime',
        'last_seen': 'datetime',
        'last_seen_entry_id': 'str',
        'manual_score': 'bool',
        'manual_set_time': 'datetime',
        'manually_edited_fields': 'list[str]',
        'modified': 'datetime',
        'score': 'int',
        'set_by': 'str',
        'sort_values': 'list[str]',
        'source': 'str',
        'timestamp': 'datetime',
        'value': 'str',
        'version': 'int'
    }

    attribute_map = {
        'custom_fields': 'CustomFields',
        'account': 'account',
        'calculated_time': 'calculatedTime',
        'comment': 'comment',
        'first_seen': 'firstSeen',
        'first_seen_entry_id': 'firstSeenEntryID',
        'id': 'id',
        'indicator_type': 'indicator_type',
        'insight_cache': 'insightCache',
        'investigation_i_ds': 'investigationIDs',
        'last_reputation_run': 'lastReputationRun',
        'last_seen': 'lastSeen',
        'last_seen_entry_id': 'lastSeenEntryID',
        'manual_score': 'manualScore',
        'manual_set_time': 'manualSetTime',
        'manually_edited_fields': 'manuallyEditedFields',
        'modified': 'modified',
        'score': 'score',
        'set_by': 'setBy',
        'sort_values': 'sortValues',
        'source': 'source',
        'timestamp': 'timestamp',
        'value': 'value',
        'version': 'version'
    }

    def __init__(self, custom_fields=None, account=None, calculated_time=None, comment=None, first_seen=None, first_seen_entry_id=None, id=None, indicator_type=None, insight_cache=None, investigation_i_ds=None, last_reputation_run=None, last_seen=None, last_seen_entry_id=None, manual_score=None, manual_set_time=None, manually_edited_fields=None, modified=None, score=None, set_by=None, sort_values=None, source=None, timestamp=None, value=None, version=None):  # noqa: E501
        """IocObject - a model defined in Swagger"""  # noqa: E501

        self._custom_fields = None
        self._account = None
        self._calculated_time = None
        self._comment = None
        self._first_seen = None
        self._first_seen_entry_id = None
        self._id = None
        self._indicator_type = None
        self._insight_cache = None
        self._investigation_i_ds = None
        self._last_reputation_run = None
        self._last_seen = None
        self._last_seen_entry_id = None
        self._manual_score = None
        self._manual_set_time = None
        self._manually_edited_fields = None
        self._modified = None
        self._score = None
        self._set_by = None
        self._sort_values = None
        self._source = None
        self._timestamp = None
        self._value = None
        self._version = None
        self.discriminator = None

        if custom_fields is not None:
            self.custom_fields = custom_fields
        if account is not None:
            self.account = account
        if calculated_time is not None:
            self.calculated_time = calculated_time
        if comment is not None:
            self.comment = comment
        if first_seen is not None:
            self.first_seen = first_seen
        if first_seen_entry_id is not None:
            self.first_seen_entry_id = first_seen_entry_id
        if id is not None:
            self.id = id
        if indicator_type is not None:
            self.indicator_type = indicator_type
        if insight_cache is not None:
            self.insight_cache = insight_cache
        if investigation_i_ds is not None:
            self.investigation_i_ds = investigation_i_ds
        if last_reputation_run is not None:
            self.last_reputation_run = last_reputation_run
        if last_seen is not None:
            self.last_seen = last_seen
        if last_seen_entry_id is not None:
            self.last_seen_entry_id = last_seen_entry_id
        if manual_score is not None:
            self.manual_score = manual_score
        if manual_set_time is not None:
            self.manual_set_time = manual_set_time
        if manually_edited_fields is not None:
            self.manually_edited_fields = manually_edited_fields
        if modified is not None:
            self.modified = modified
        if score is not None:
            self.score = score
        if set_by is not None:
            self.set_by = set_by
        if sort_values is not None:
            self.sort_values = sort_values
        if source is not None:
            self.source = source
        if timestamp is not None:
            self.timestamp = timestamp
        if value is not None:
            self.value = value
        if version is not None:
            self.version = version

    @property
    def custom_fields(self):
        """Gets the custom_fields of this IocObject.  # noqa: E501


        :return: The custom_fields of this IocObject.  # noqa: E501
        :rtype: CustomFields
        """
        return self._custom_fields

    @custom_fields.setter
    def custom_fields(self, custom_fields):
        """Sets the custom_fields of this IocObject.


        :param custom_fields: The custom_fields of this IocObject.  # noqa: E501
        :type: CustomFields
        """

        self._custom_fields = custom_fields

    @property
    def account(self):
        """Gets the account of this IocObject.  # noqa: E501


        :return: The account of this IocObject.  # noqa: E501
        :rtype: str
        """
        return self._account

    @account.setter
    def account(self, account):
        """Sets the account of this IocObject.


        :param account: The account of this IocObject.  # noqa: E501
        :type: str
        """

        self._account = account

    @property
    def calculated_time(self):
        """Gets the calculated_time of this IocObject.  # noqa: E501

        Do not set the fields bellow this line  # noqa: E501

        :return: The calculated_time of this IocObject.  # noqa: E501
        :rtype: datetime
        """
        return self._calculated_time

    @calculated_time.setter
    def calculated_time(self, calculated_time):
        """Sets the calculated_time of this IocObject.

        Do not set the fields bellow this line  # noqa: E501

        :param calculated_time: The calculated_time of this IocObject.  # noqa: E501
        :type: datetime
        """

        self._calculated_time = calculated_time

    @property
    def comment(self):
        """Gets the comment of this IocObject.  # noqa: E501


        :return: The comment of this IocObject.  # noqa: E501
        :rtype: str
        """
        return self._comment

    @comment.setter
    def comment(self, comment):
        """Sets the comment of this IocObject.


        :param comment: The comment of this IocObject.  # noqa: E501
        :type: str
        """

        self._comment = comment

    @property
    def first_seen(self):
        """Gets the first_seen of this IocObject.  # noqa: E501


        :return: The first_seen of this IocObject.  # noqa: E501
        :rtype: datetime
        """
        return self._first_seen

    @first_seen.setter
    def first_seen(self, first_seen):
        """Sets the first_seen of this IocObject.


        :param first_seen: The first_seen of this IocObject.  # noqa: E501
        :type: datetime
        """

        self._first_seen = first_seen

    @property
    def first_seen_entry_id(self):
        """Gets the first_seen_entry_id of this IocObject.  # noqa: E501


        :return: The first_seen_entry_id of this IocObject.  # noqa: E501
        :rtype: str
        """
        return self._first_seen_entry_id

    @first_seen_entry_id.setter
    def first_seen_entry_id(self, first_seen_entry_id):
        """Sets the first_seen_entry_id of this IocObject.


        :param first_seen_entry_id: The first_seen_entry_id of this IocObject.  # noqa: E501
        :type: str
        """

        self._first_seen_entry_id = first_seen_entry_id

    @property
    def id(self):
        """Gets the id of this IocObject.  # noqa: E501


        :return: The id of this IocObject.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this IocObject.


        :param id: The id of this IocObject.  # noqa: E501
        :type: str
        """

        self._id = id

    @property
    def indicator_type(self):
        """Gets the indicator_type of this IocObject.  # noqa: E501


        :return: The indicator_type of this IocObject.  # noqa: E501
        :rtype: str
        """
        return self._indicator_type

    @indicator_type.setter
    def indicator_type(self, indicator_type):
        """Sets the indicator_type of this IocObject.


        :param indicator_type: The indicator_type of this IocObject.  # noqa: E501
        :type: str
        """

        self._indicator_type = indicator_type

    @property
    def insight_cache(self):
        """Gets the insight_cache of this IocObject.  # noqa: E501


        :return: The insight_cache of this IocObject.  # noqa: E501
        :rtype: InsightCache
        """
        return self._insight_cache

    @insight_cache.setter
    def insight_cache(self, insight_cache):
        """Sets the insight_cache of this IocObject.


        :param insight_cache: The insight_cache of this IocObject.  # noqa: E501
        :type: InsightCache
        """

        self._insight_cache = insight_cache

    @property
    def investigation_i_ds(self):
        """Gets the investigation_i_ds of this IocObject.  # noqa: E501


        :return: The investigation_i_ds of this IocObject.  # noqa: E501
        :rtype: list[str]
        """
        return self._investigation_i_ds

    @investigation_i_ds.setter
    def investigation_i_ds(self, investigation_i_ds):
        """Sets the investigation_i_ds of this IocObject.


        :param investigation_i_ds: The investigation_i_ds of this IocObject.  # noqa: E501
        :type: list[str]
        """

        self._investigation_i_ds = investigation_i_ds

    @property
    def last_reputation_run(self):
        """Gets the last_reputation_run of this IocObject.  # noqa: E501


        :return: The last_reputation_run of this IocObject.  # noqa: E501
        :rtype: datetime
        """
        return self._last_reputation_run

    @last_reputation_run.setter
    def last_reputation_run(self, last_reputation_run):
        """Sets the last_reputation_run of this IocObject.


        :param last_reputation_run: The last_reputation_run of this IocObject.  # noqa: E501
        :type: datetime
        """

        self._last_reputation_run = last_reputation_run

    @property
    def last_seen(self):
        """Gets the last_seen of this IocObject.  # noqa: E501


        :return: The last_seen of this IocObject.  # noqa: E501
        :rtype: datetime
        """
        return self._last_seen

    @last_seen.setter
    def last_seen(self, last_seen):
        """Sets the last_seen of this IocObject.


        :param last_seen: The last_seen of this IocObject.  # noqa: E501
        :type: datetime
        """

        self._last_seen = last_seen

    @property
    def last_seen_entry_id(self):
        """Gets the last_seen_entry_id of this IocObject.  # noqa: E501


        :return: The last_seen_entry_id of this IocObject.  # noqa: E501
        :rtype: str
        """
        return self._last_seen_entry_id

    @last_seen_entry_id.setter
    def last_seen_entry_id(self, last_seen_entry_id):
        """Sets the last_seen_entry_id of this IocObject.


        :param last_seen_entry_id: The last_seen_entry_id of this IocObject.  # noqa: E501
        :type: str
        """

        self._last_seen_entry_id = last_seen_entry_id

    @property
    def manual_score(self):
        """Gets the manual_score of this IocObject.  # noqa: E501


        :return: The manual_score of this IocObject.  # noqa: E501
        :rtype: bool
        """
        return self._manual_score

    @manual_score.setter
    def manual_score(self, manual_score):
        """Sets the manual_score of this IocObject.


        :param manual_score: The manual_score of this IocObject.  # noqa: E501
        :type: bool
        """

        self._manual_score = manual_score

    @property
    def manual_set_time(self):
        """Gets the manual_set_time of this IocObject.  # noqa: E501


        :return: The manual_set_time of this IocObject.  # noqa: E501
        :rtype: datetime
        """
        return self._manual_set_time

    @manual_set_time.setter
    def manual_set_time(self, manual_set_time):
        """Sets the manual_set_time of this IocObject.


        :param manual_set_time: The manual_set_time of this IocObject.  # noqa: E501
        :type: datetime
        """

        self._manual_set_time = manual_set_time

    @property
    def manually_edited_fields(self):
        """Gets the manually_edited_fields of this IocObject.  # noqa: E501


        :return: The manually_edited_fields of this IocObject.  # noqa: E501
        :rtype: list[str]
        """
        return self._manually_edited_fields

    @manually_edited_fields.setter
    def manually_edited_fields(self, manually_edited_fields):
        """Sets the manually_edited_fields of this IocObject.


        :param manually_edited_fields: The manually_edited_fields of this IocObject.  # noqa: E501
        :type: list[str]
        """

        self._manually_edited_fields = manually_edited_fields

    @property
    def modified(self):
        """Gets the modified of this IocObject.  # noqa: E501


        :return: The modified of this IocObject.  # noqa: E501
        :rtype: datetime
        """
        return self._modified

    @modified.setter
    def modified(self, modified):
        """Sets the modified of this IocObject.


        :param modified: The modified of this IocObject.  # noqa: E501
        :type: datetime
        """

        self._modified = modified

    @property
    def score(self):
        """Gets the score of this IocObject.  # noqa: E501


        :return: The score of this IocObject.  # noqa: E501
        :rtype: int
        """
        return self._score

    @score.setter
    def score(self, score):
        """Sets the score of this IocObject.


        :param score: The score of this IocObject.  # noqa: E501
        :type: int
        """

        self._score = score

    @property
    def set_by(self):
        """Gets the set_by of this IocObject.  # noqa: E501


        :return: The set_by of this IocObject.  # noqa: E501
        :rtype: str
        """
        return self._set_by

    @set_by.setter
    def set_by(self, set_by):
        """Sets the set_by of this IocObject.


        :param set_by: The set_by of this IocObject.  # noqa: E501
        :type: str
        """

        self._set_by = set_by

    @property
    def sort_values(self):
        """Gets the sort_values of this IocObject.  # noqa: E501


        :return: The sort_values of this IocObject.  # noqa: E501
        :rtype: list[str]
        """
        return self._sort_values

    @sort_values.setter
    def sort_values(self, sort_values):
        """Sets the sort_values of this IocObject.


        :param sort_values: The sort_values of this IocObject.  # noqa: E501
        :type: list[str]
        """

        self._sort_values = sort_values

    @property
    def source(self):
        """Gets the source of this IocObject.  # noqa: E501


        :return: The source of this IocObject.  # noqa: E501
        :rtype: str
        """
        return self._source

    @source.setter
    def source(self, source):
        """Sets the source of this IocObject.


        :param source: The source of this IocObject.  # noqa: E501
        :type: str
        """

        self._source = source

    @property
    def timestamp(self):
        """Gets the timestamp of this IocObject.  # noqa: E501


        :return: The timestamp of this IocObject.  # noqa: E501
        :rtype: datetime
        """
        return self._timestamp

    @timestamp.setter
    def timestamp(self, timestamp):
        """Sets the timestamp of this IocObject.


        :param timestamp: The timestamp of this IocObject.  # noqa: E501
        :type: datetime
        """

        self._timestamp = timestamp

    @property
    def value(self):
        """Gets the value of this IocObject.  # noqa: E501


        :return: The value of this IocObject.  # noqa: E501
        :rtype: str
        """
        return self._value

    @value.setter
    def value(self, value):
        """Sets the value of this IocObject.


        :param value: The value of this IocObject.  # noqa: E501
        :type: str
        """

        self._value = value

    @property
    def version(self):
        """Gets the version of this IocObject.  # noqa: E501


        :return: The version of this IocObject.  # noqa: E501
        :rtype: int
        """
        return self._version

    @version.setter
    def version(self, version):
        """Sets the version of this IocObject.


        :param version: The version of this IocObject.  # noqa: E501
        :type: int
        """

        self._version = version

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(IocObject, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, IocObject):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Demisto API

    This is the public REST API to integrate with the demisto server. HTTP request can be sent using any HTTP-client.  For an example dedicated client take a look at: https://github.com/demisto/demisto-py.  Requests must include API-key that can be generated in the Demisto web client under 'Settings' -> 'Integrations' -> 'API keys'   Optimistic Locking and Versioning\\:  When using Demisto REST API, you will need to make sure to work on the latest version of the item (incident, entry, etc.), otherwise, you will get a DB version error (which not allow you to override a newer item). In addition, you can pass 'version\\: -1' to force data override (make sure that other users data might be lost).  Assume that Alice and Bob both read the same data from Demisto server, then they both changed the data, and then both tried to write the new versions back to the server. Whose changes should be saved? Alice’s? Bob’s? To solve this, each data item in Demisto has a numeric incremental version. If Alice saved an item with version 4 and Bob trying to save the same item with version 3, Demisto will rollback Bob request and returns a DB version conflict error. Bob will need to get the latest item and work on it so Alice work will not get lost.  Example request using 'curl'\\:  ``` curl 'https://hostname:443/incidents/search' -H 'content-type: application/json' -H 'accept: application/json' -H 'Authorization: <API Key goes here>' --data-binary '{\"filter\":{\"query\":\"-status:closed -category:job\",\"period\":{\"by\":\"day\",\"fromValue\":7}}}' --compressed ```  # noqa: E501

    OpenAPI spec version: 2.0.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six

from demisto_client.demisto_api.models.command import Command  # noqa: F401,E501
from demisto_client.demisto_api.models.script_sub_type import ScriptSubType  # noqa: F401,E501
from demisto_client.demisto_api.models.script_type import ScriptType  # noqa: F401,E501


class IntegrationScript(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'commands': 'list[Command]',
        'docker_image': 'str',
        'feed': 'bool',
        'is_fetch': 'bool',
        'is_fetch_credentials': 'bool',
        'long_running': 'bool',
        'long_running_port_mapping': 'bool',
        'run_once': 'bool',
        'script': 'str',
        'subtype': 'ScriptSubType',
        'type': 'ScriptType'
    }

    attribute_map = {
        'commands': 'commands',
        'docker_image': 'dockerImage',
        'feed': 'feed',
        'is_fetch': 'isFetch',
        'is_fetch_credentials': 'isFetchCredentials',
        'long_running': 'longRunning',
        'long_running_port_mapping': 'longRunningPortMapping',
        'run_once': 'runOnce',
        'script': 'script',
        'subtype': 'subtype',
        'type': 'type'
    }

    def __init__(self, commands=None, docker_image=None, feed=None, is_fetch=None, is_fetch_credentials=None, long_running=None, long_running_port_mapping=None, run_once=None, script=None, subtype=None, type=None):  # noqa: E501
        """IntegrationScript - a model defined in Swagger"""  # noqa: E501

        self._commands = None
        self._docker_image = None
        self._feed = None
        self._is_fetch = None
        self._is_fetch_credentials = None
        self._long_running = None
        self._long_running_port_mapping = None
        self._run_once = None
        self._script = None
        self._subtype = None
        self._type = None
        self.discriminator = None

        if commands is not None:
            self.commands = commands
        if docker_image is not None:
            self.docker_image = docker_image
        if feed is not None:
            self.feed = feed
        if is_fetch is not None:
            self.is_fetch = is_fetch
        if is_fetch_credentials is not None:
            self.is_fetch_credentials = is_fetch_credentials
        if long_running is not None:
            self.long_running = long_running
        if long_running_port_mapping is not None:
            self.long_running_port_mapping = long_running_port_mapping
        if run_once is not None:
            self.run_once = run_once
        if script is not None:
            self.script = script
        if subtype is not None:
            self.subtype = subtype
        if type is not None:
            self.type = type

    @property
    def commands(self):
        """Gets the commands of this IntegrationScript.  # noqa: E501


        :return: The commands of this IntegrationScript.  # noqa: E501
        :rtype: list[Command]
        """
        return self._commands

    @commands.setter
    def commands(self, commands):
        """Sets the commands of this IntegrationScript.


        :param commands: The commands of this IntegrationScript.  # noqa: E501
        :type: list[Command]
        """

        self._commands = commands

    @property
    def docker_image(self):
        """Gets the docker_image of this IntegrationScript.  # noqa: E501


        :return: The docker_image of this IntegrationScript.  # noqa: E501
        :rtype: str
        """
        return self._docker_image

    @docker_image.setter
    def docker_image(self, docker_image):
        """Sets the docker_image of this IntegrationScript.


        :param docker_image: The docker_image of this IntegrationScript.  # noqa: E501
        :type: str
        """

        self._docker_image = docker_image

    @property
    def feed(self):
        """Gets the feed of this IntegrationScript.  # noqa: E501


        :return: The feed of this IntegrationScript.  # noqa: E501
        :rtype: bool
        """
        return self._feed

    @feed.setter
    def feed(self, feed):
        """Sets the feed of this IntegrationScript.


        :param feed: The feed of this IntegrationScript.  # noqa: E501
        :type: bool
        """

        self._feed = feed

    @property
    def is_fetch(self):
        """Gets the is_fetch of this IntegrationScript.  # noqa: E501


        :return: The is_fetch of this IntegrationScript.  # noqa: E501
        :rtype: bool
        """
        return self._is_fetch

    @is_fetch.setter
    def is_fetch(self, is_fetch):
        """Sets the is_fetch of this IntegrationScript.


        :param is_fetch: The is_fetch of this IntegrationScript.  # noqa: E501
        :type: bool
        """

        self._is_fetch = is_fetch

    @property
    def is_fetch_credentials(self):
        """Gets the is_fetch_credentials of this IntegrationScript.  # noqa: E501


        :return: The is_fetch_credentials of this IntegrationScript.  # noqa: E501
        :rtype: bool
        """
        return self._is_fetch_credentials

    @is_fetch_credentials.setter
    def is_fetch_credentials(self, is_fetch_credentials):
        """Sets the is_fetch_credentials of this IntegrationScript.


        :param is_fetch_credentials: The is_fetch_credentials of this IntegrationScript.  # noqa: E501
        :type: bool
        """

        self._is_fetch_credentials = is_fetch_credentials

    @property
    def long_running(self):
        """Gets the long_running of this IntegrationScript.  # noqa: E501


        :return: The long_running of this IntegrationScript.  # noqa: E501
        :rtype: bool
        """
        return self._long_running

    @long_running.setter
    def long_running(self, long_running):
        """Sets the long_running of this IntegrationScript.


        :param long_running: The long_running of this IntegrationScript.  # noqa: E501
        :type: bool
        """

        self._long_running = long_running

    @property
    def long_running_port_mapping(self):
        """Gets the long_running_port_mapping of this IntegrationScript.  # noqa: E501


        :return: The long_running_port_mapping of this IntegrationScript.  # noqa: E501
        :rtype: bool
        """
        return self._long_running_port_mapping

    @long_running_port_mapping.setter
    def long_running_port_mapping(self, long_running_port_mapping):
        """Sets the long_running_port_mapping of this IntegrationScript.


        :param long_running_port_mapping: The long_running_port_mapping of this IntegrationScript.  # noqa: E501
        :type: bool
        """

        self._long_running_port_mapping = long_running_port_mapping

    @property
    def run_once(self):
        """Gets the run_once of this IntegrationScript.  # noqa: E501


        :return: The run_once of this IntegrationScript.  # noqa: E501
        :rtype: bool
        """
        return self._run_once

    @run_once.setter
    def run_once(self, run_once):
        """Sets the run_once of this IntegrationScript.


        :param run_once: The run_once of this IntegrationScript.  # noqa: E501
        :type: bool
        """

        self._run_once = run_once

    @property
    def script(self):
        """Gets the script of this IntegrationScript.  # noqa: E501


        :return: The script of this IntegrationScript.  # noqa: E501
        :rtype: str
        """
        return self._script

    @script.setter
    def script(self, script):
        """Sets the script of this IntegrationScript.


        :param script: The script of this IntegrationScript.  # noqa: E501
        :type: str
        """

        self._script = script

    @property
    def subtype(self):
        """Gets the subtype of this IntegrationScript.  # noqa: E501


        :return: The subtype of this IntegrationScript.  # noqa: E501
        :rtype: ScriptSubType
        """
        return self._subtype

    @subtype.setter
    def subtype(self, subtype):
        """Sets the subtype of this IntegrationScript.


        :param subtype: The subtype of this IntegrationScript.  # noqa: E501
        :type: ScriptSubType
        """

        self._subtype = subtype

    @property
    def type(self):
        """Gets the type of this IntegrationScript.  # noqa: E501


        :return: The type of this IntegrationScript.  # noqa: E501
        :rtype: ScriptType
        """
        return self._type

    @type.setter
    def type(self, type):
        """Sets the type of this IntegrationScript.


        :param type: The type of this IntegrationScript.  # noqa: E501
        :type: ScriptType
        """

        self._type = type

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(IntegrationScript, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, IntegrationScript):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Demisto API

    This is the public REST API to integrate with the demisto server. HTTP request can be sent using any HTTP-client.  For an example dedicated client take a look at: https://github.com/demisto/demisto-py.  Requests must include API-key that can be generated in the Demisto web client under 'Settings' -> 'Integrations' -> 'API keys'   Optimistic Locking and Versioning\\:  When using Demisto REST API, you will need to make sure to work on the latest version of the item (incident, entry, etc.), otherwise, you will get a DB version error (which not allow you to override a newer item). In addition, you can pass 'version\\: -1' to force data override (make sure that other users data might be lost).  Assume that Alice and Bob both read the same data from Demisto server, then they both changed the data, and then both tried to write the new versions back to the server. Whose changes should be saved? Alice’s? Bob’s? To solve this, each data item in Demisto has a numeric incremental version. If Alice saved an item with version 4 and Bob trying to save the same item with version 3, Demisto will rollback Bob request and returns a DB version conflict error. Bob will need to get the latest item and work on it so Alice work will not get lost.  Example request using 'curl'\\:  ``` curl 'https://hostname:443/incidents/search' -H 'content-type: application/json' -H 'accept: application/json' -H 'Authorization: <API Key goes here>' --data-binary '{\"filter\":{\"query\":\"-status:closed -category:job\",\"period\":{\"by\":\"day\",\"fromValue\":7}}}' --compressed ```  # noqa: E501

    OpenAPI spec version: 2.0.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six

from demisto_client.demisto_api.models.advance_arg import AdvanceArg  # noqa: F401,E501
from demisto_client.demisto_api.models.grid_column import GridColumn  # noqa: F401,E501


class Question(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'columns': 'list[GridColumn]',
        'default_rows': 'list[dict(str, object)]',
        'field_associated': 'str',
        'id': 'str',
        'label': 'str',
        'label_arg': 'AdvanceArg',
        'options': 'list[str]',
        'placeholder': 'str',
        'required': 'bool',
        'tooltip': 'str',
        'type': 'str'
    }

    attribute_map = {
        'columns': 'columns',
        'default_rows': 'defaultRows',
        'field_associated': 'fieldAssociated',
        'id': 'id',
        'label': 'label',
        'label_arg': 'labelArg',
        'options': 'options',
        'placeholder': 'placeholder',
        'required': 'required',
        'tooltip': 'tooltip',
        'type': 'type'
    }

    def __init__(self, columns=None, default_rows=None, field_associated=None, id=None, label=None, label_arg=None, options=None, placeholder=None, required=None, tooltip=None, type=None):  # noqa: E501
        """Question - a model defined in Swagger"""  # noqa: E501

        self._columns = None
        self._default_rows = None
        self._field_associated = None
        self._id = None
        self._label = None
        self._label_arg = None
        self._options = None
        self._placeholder = None
        self._required = None
        self._tooltip = None
        self._type = None
        self.discriminator = None

        if columns is not None:
            self.columns = columns
        if default_rows is not None:
            self.default_rows = default_rows
        if field_associated is not None:
            self.field_associated = field_associated
        if id is not None:
            self.id = id
        if label is not None:
            self.label = label
        if label_arg is not None:
            self.label_arg = label_arg
        if options is not None:
            self.options = options
        if placeholder is not None:
            self.placeholder = placeholder
        if required is not None:
            self.required = required
        if tooltip is not None:
            self.tooltip = tooltip
        if type is not None:
            self.type = type

    @property
    def columns(self):
        """Gets the columns of this Question.  # noqa: E501


        :return: The columns of this Question.  # noqa: E501
        :rtype: list[GridColumn]
        """
        return self._columns

    @columns.setter
    def columns(self, columns):
        """Sets the columns of this Question.


        :param columns: The columns of this Question.  # noqa: E501
        :type: list[GridColumn]
        """

        self._columns = columns

    @property
    def default_rows(self):
        """Gets the default_rows of this Question.  # noqa: E501


        :return: The default_rows of this Question.  # noqa: E501
        :rtype: list[dict(str, object)]
        """
        return self._default_rows

    @default_rows.setter
    def default_rows(self, default_rows):
        """Sets the default_rows of this Question.


        :param default_rows: The default_rows of this Question.  # noqa: E501
        :type: list[dict(str, object)]
        """

        self._default_rows = default_rows

    @property
    def field_associated(self):
        """Gets the field_associated of this Question.  # noqa: E501


        :return: The field_associated of this Question.  # noqa: E501
        :rtype: str
        """
        return self._field_associated

    @field_associated.setter
    def field_associated(self, field_associated):
        """Sets the field_associated of this Question.


        :param field_associated: The field_associated of this Question.  # noqa: E501
        :type: str
        """

        self._field_associated = field_associated

    @property
    def id(self):
        """Gets the id of this Question.  # noqa: E501


        :return: The id of this Question.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this Question.


        :param id: The id of this Question.  # noqa: E501
        :type: str
        """

        self._id = id

    @property
    def label(self):
        """Gets the label of this Question.  # noqa: E501


        :return: The label of this Question.  # noqa: E501
        :rtype: str
        """
        return self._label

    @label.setter
    def label(self, label):
        """Sets the label of this Question.


        :param label: The label of this Question.  # noqa: E501
        :type: str
        """

        self._label = label

    @property
    def label_arg(self):
        """Gets the label_arg of this Question.  # noqa: E501


        :return: The label_arg of this Question.  # noqa: E501
        :rtype: AdvanceArg
        """
        return self._label_arg

    @label_arg.setter
    def label_arg(self, label_arg):
        """Sets the label_arg of this Question.


        :param label_arg: The label_arg of this Question.  # noqa: E501
        :type: AdvanceArg
        """

        self._label_arg = label_arg

    @property
    def options(self):
        """Gets the options of this Question.  # noqa: E501


        :return: The options of this Question.  # noqa: E501
        :rtype: list[str]
        """
        return self._options

    @options.setter
    def options(self, options):
        """Sets the options of this Question.


        :param options: The options of this Question.  # noqa: E501
        :type: list[str]
        """

        self._options = options

    @property
    def placeholder(self):
        """Gets the placeholder of this Question.  # noqa: E501


        :return: The placeholder of this Question.  # noqa: E501
        :rtype: str
        """
        return self._placeholder

    @placeholder.setter
    def placeholder(self, placeholder):
        """Sets the placeholder of this Question.


        :param placeholder: The placeholder of this Question.  # noqa: E501
        :type: str
        """

        self._placeholder = placeholder

    @property
    def required(self):
        """Gets the required of this Question.  # noqa: E501


        :return: The required of this Question.  # noqa: E501
        :rtype: bool
        """
        return self._required

    @required.setter
    def required(self, required):
        """Sets the required of this Question.


        :param required: The required of this Question.  # noqa: E501
        :type: bool
        """

        self._required = required

    @property
    def tooltip(self):
        """Gets the tooltip of this Question.  # noqa: E501


        :return: The tooltip of this Question.  # noqa: E501
        :rtype: str
        """
        return self._tooltip

    @tooltip.setter
    def tooltip(self, tooltip):
        """Sets the tooltip of this Question.


        :param tooltip: The tooltip of this Question.  # noqa: E501
        :type: str
        """

        self._tooltip = tooltip

    @property
    def type(self):
        """Gets the type of this Question.  # noqa: E501


        :return: The type of this Question.  # noqa: E501
        :rtype: str
        """
        return self._type

    @type.setter
    def type(self, type):
        """Sets the type of this Question.


        :param type: The type of this Question.  # noqa: E501
        :type: str
        """

        self._type = type

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(Question, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, Question):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



import os
import unittest
import pytest
from urllib3_mock import Responses
import demisto_client
from datetime import datetime
import json
import urllib3
import tempfile
from demisto_client.demisto_api import rest

responses = Responses('requests.packages.urllib3')

api_key = 'sample_api_key'
host = 'http://localhost:8080'


def assert_reset():
    assert len(responses._urls) == 0
    assert len(responses.calls) == 0


def get_login_mocker(mocker, num_of_http_requests_mock: int=0):
    from urllib3 import PoolManager

    raw_http_responses = [
        urllib3.response.HTTPResponse(body=b'{}', status=200, headers={'Set-Cookie': 'XSRF-TOKEN=1234;'}),
        urllib3.response.HTTPResponse(body=b'{}', status=200, headers={'Set-Cookie': '1234'}),
    ]

    for i in range(num_of_http_requests_mock):
        raw_http_responses.append(
            urllib3.response.HTTPResponse(body=b'{}', status=200)
        )

    return mocker.patch.object(PoolManager, 'request', side_effect=raw_http_responses)


def test_get_docker_images():
    '''Check GET docker images.'''

    @responses.activate
    def run():
        body = '{ "images": [{"id": "aae7b8aaba8c", "repository": ' \
               '"openapitools/openapi-generator-cli", "tag": "latest", "createdSince": "3 days ago", ' \
               '"createdAt": "2019-08-19 13:34:22 +0300 IDT", "size": "131MB" }]}'
        responses.add('GET', '/settings/docker-images',
                      body=body,
                      status=200,
                      content_type='application/json')

        # create an instance of the API class
        api_instance = demisto_client.configure(base_url=host, api_key=api_key)
        api_response = api_instance.get_docker_images()

        assert api_response.images[0].created_at == '2019-08-19 13:34:22 +0300 IDT'
        assert api_response.images[0].id == 'aae7b8aaba8c'
        assert api_response.images[0].repository == 'openapitools/openapi-generator-cli'

        assert len(responses.calls) == 1
        assert responses.calls[0].request.url == '/settings/docker-images'
        assert responses.calls[0].request.host == 'localhost'
        assert responses.calls[0].request.scheme == 'http'

    run()
    assert_reset()


def test_create_incident():
    '''Check creating an incident.'''

    @responses.activate
    def run():
        body = '{"name":"Test Incident","owner":"Admin","parent":"","phase":"",' \
               '"playbookId":"playbook0","playbook_id":"playbook0","rawCategory":"",' \
               '"rawCloseReason":"","rawJSON":"","rawName":"Test Incident","rawPhase":"",' \
               '"rawType":"Unclassified","raw_category":"","raw_close_reason":"","raw_json":"",' \
               '"raw_name":"Test Incident","raw_phase":"","raw_type":"Unclassified","reason":"",' \
               '"runStatus":"","run_status":"","severity":0,"sourceBrand":"Manual",' \
               '"sourceInstance":"admin","source_brand":"Manual","source_instance":"admin",' \
               '"status":0,"type":"Unclassified","version":1}'
        responses.add('POST', '/incident',
                      body=body,
                      status=200,
                      content_type='application/json')

        # create an instance of the API class
        api_instance = demisto_client.configure(base_url=host, api_key=api_key)
        create_incident_request = demisto_client.demisto_api.CreateIncidentRequest()
        create_incident_request.name = 'Test Incident'
        create_incident_request.type = 'Unclassified'
        create_incident_request.owner = 'Admin'
        create_incident_request.occurred = datetime.now()
        api_response = api_instance.create_incident(create_incident_request=create_incident_request)

        assert api_response.name == 'Test Incident'
        assert api_response.type == 'Unclassified'
        assert api_response.owner == 'Admin'

        assert len(responses.calls) == 1
        req = responses.calls[0].request
        assert req.url == '/incident'
        assert req.host == 'localhost'
        assert req.scheme == 'http'
        # veriy date field occurred according to rfc 3339
        req_body = json.loads(req.body)
        assert req_body['occurred'][-6] == '+' or req_body['occurred'][-6] == '-'  # end with +/- offset 

    run()
    assert_reset()


def test_get_reports():
    '''Testing GET all reports.'''

    @responses.activate
    def run():
        body = '[{"created_by":"DBot","dashboard":"None","decoder":{},"description":"This report ' \
               'generates Mean Time to Resolve by Incident type for last 2 Quarters",' \
               '"id":"MTTRbyIncidentType2Quar","locked":false,"name":"Mean time to Resolve by ' \
               'Incident Type (Last 2 Quarters)","orientation":"portrait","prev_name":"Mean time to ' \
               'Resolve by Incident Type (Last 2 Quarters)","prev_type":"pdf","type":"pdf",' \
               '"version":4}]'

        responses.add('GET', '/reports',
                      body=body,
                      status=200,
                      content_type='application/json')

        api_instance = demisto_client.configure(base_url=host, api_key=api_key, debug=False)
        api_response = api_instance.get_all_reports()

        assert api_response[0].id == 'MTTRbyIncidentType2Quar'
        assert api_response[
                   0].description == 'This report generates Mean Time to Resolve by Incident type for ' \
                                     '' \
                                     'last 2 Quarters'
        assert api_response[0].version == 4

        assert len(responses.calls) == 1
        assert responses.calls[0].request.url == '/reports'
        assert responses.calls[0].request.host == 'localhost'
        assert responses.calls[0].request.scheme == 'http'

    run()
    assert_reset()


def test_indicators_search():
    '''Testing search for indicator.'''

    @responses.activate
    def run():
        body = r'''
        {
        "iocObjects": [{
            "id": "4737",
            "version": 1,
            "modified": "2019-07-14T16:54:02.719044+03:00",
            "account": "",
            "timestamp": "2019-07-14T16:54:02.718422+03:00",
            "indicator_type": "IP",
            "value": "92.63.197.153",
            "source": "Recorded Future",
            "investigationIDs": ["1750"],
            "lastSeen": "2019-07-14T16:54:02.718378+03:00",
            "firstSeen": "2019-07-14T16:54:02.718378+03:00",
            "lastSeenEntryID": "API",
            "firstSeenEntryID": "API",
            "score": 3,
            "manualScore": true,
            "setBy": "DBotWeak",
            "manualSetTime": "0001-01-01T00:00:00Z",
            "insightCache": null,
            "calculatedTime": "2019-07-14T16:54:02.718378+03:00",
            "lastReputationRun": "0001-01-01T00:00:00Z",
            "comment": "From Recorded Future risk list, Score - 89",
            "manuallyEditedFields": null
        }],
        "total": 1
    }
    '''

        responses.add('POST', '/indicators/search',
                      body=body,
                      status=200,
                      content_type='application/json')
        api_instance = demisto_client.configure(base_url=host, api_key=api_key, debug=False)
        indicator_filter = demisto_client.demisto_api.IndicatorFilter()  # IndicatorFilter |  (optional)
        indicator_filter.query = 'value:92.63.197.153'
        api_response = api_instance.indicators_search(indicator_filter=indicator_filter)

        assert api_response.ioc_objects[0].get('comment') == 'From Recorded Future risk list, Score - 89'
        # assert api_response.type == 'Unclassified'
        # assert api_response.owner == 'Admin'

        assert len(responses.calls) == 1
        assert responses.calls[0].request.url == '/indicators/search'
        assert responses.calls[0].request.host == 'localhost'
        assert responses.calls[0].request.scheme == 'http'

    run()
    assert_reset()


def test_export_entry():
    '''Testing export entry artifact.'''

    @responses.activate
    def run():
        body = "entry_artifact_6@1770.md"
        responses.add('POST', '/entry/exportArtifact',
                      body=body,
                      status=200,
                      content_type='application/json')
        api_instance = demisto_client.configure(base_url=host, api_key=api_key, debug=False)
        download_entry = demisto_client.demisto_api.DownloadEntry()  # DownloadEntry |  (optional)

        download_entry.id = '6@1770'
        download_entry.investigation_id = '1770'

        api_result = api_instance.entry_export_artifact(download_entry=download_entry)

        assert api_result == 'entry_artifact_6@1770.md'

        assert len(responses.calls) == 1
        assert responses.calls[0].request.url == '/entry/exportArtifact'
        assert responses.calls[0].request.host == 'localhost'
        assert responses.calls[0].request.scheme == 'http'

    run()
    assert_reset()


def test_generic_request():
    '''Testing generic requst.'''

    @responses.activate
    def run():
        responses.add('POST', '/test',
                      body="all good",
                      status=200,
                      content_type='text/plain')
        api_instance = demisto_client.configure(base_url=host, api_key=api_key, debug=False)
        (res, code, headers) = api_instance.generic_request('/test', 'POST', body="this is a test",
                                                            content_type='text/plain', accept='text/plain')

        assert res == 'all good'
        assert code == 200

        assert len(responses.calls) == 1
        assert responses.calls[0].request.url == '/test'
        assert responses.calls[0].request.host == 'localhost'
        assert responses.calls[0].request.scheme == 'http'

    run()
    assert_reset()


def test_import_layout(mocker):
    """
    Given:
        - Path for a layoutscontainer.
    When
        - Using client.import_layout() to upload a new layout.
    Then
        - The layout is being uploaded and getting back the layout metadata.
    """
    client = demisto_client.configure(base_url=host, api_key=api_key, debug=False,
                                      verify_ssl=False)
    mocker.patch.object(client, 'import_layout_with_http_info', return_value={'test': 'test'})
    res = client.import_layout('tests_data/layoutscontainer-test.json')
    assert res.get('test') == 'test'


def test_import_layout_with_http_info_with_knowing_server_version(mocker):
    """
    Given:
        - path for a layoutscontainer.
    When
        - Succeeding in getting demisto version using 'get_layouts_url_for_demisto_version' function
    Then
        - The layout is being uploaded and getting back the layout metadata.
    """
    client = demisto_client.configure(base_url=host, api_key=api_key, debug=False,
                                      verify_ssl=False)
    mocker.patch.object(client.api_client, 'call_api', side_effect=[("{'demistoVersion': '6.0.0'}", 200, {'Content-type': 'application/json'}),  {'test': 'test'}])
    res = client.import_layout('tests_data/layoutscontainer-test.json')
    assert res.get('test') == 'test'


def test_import_layout_with_http_info_without_knowing_server_version(mocker):
    """
    Given:
        - Path for a layoutscontainer.
    When
        - Not knowing demisto version, and working with 6.0.0 or higher
    Then
        - The layout is being uploaded and getting back the layout metadata.
    """
    client = demisto_client.configure(base_url=host, api_key=api_key, debug=False,
                                      verify_ssl=False)
    mocker.patch.object(client.api_client, 'call_api', side_effect=[("{'demistoVersion': '6.0.0'}", 404, {'Content-type': 'application/json'}),  {'test': 'test'}])
    res = client.import_layout('tests_data/layoutscontainer-test.json')
    assert res.get('test') == 'test'


def test_import_layout_with_http_info_with_old_layout_format(mocker):
    """
    Given:
        - Path for a layout
    When
        - Working with 5.0.0 and uploading a layout(the old version)
    Then
        - The layout is being uploaded and getting back the layout metadata.
    """
    client = demisto_client.configure(base_url=host, api_key=api_key, debug=False,
                                      verify_ssl=False)

    mocker.patch.object(client.api_client, 'call_api', side_effect=[("{'demistoVersion': '5.0.0'}", 200, {'Content-type': 'application/json'}),  {'test': 'test'}])
    res = client.import_layout('tests_data/layout-details-test-V2.json')
    assert res.get('test') == 'test'


class TestFailedGenericRequestNoEnv(unittest.TestCase):

    def test_generic_request(self):
        """
        Given:
        - Request which should result in an ApiException
        When:
        - No environment variable has been set
        Then:
        - Return ApiException without the headers in the error
        """
        from demisto_client.demisto_api.rest import ApiException

        @responses.activate
        def run():
            responses.add('POST', '/test',
                          body="Not good",
                          status=400,
                          content_type='text/plain')
            api_instance = demisto_client.configure(base_url=host, api_key=api_key, debug=False)

            with self.assertRaises(ApiException) as context:
                (_, _, _) = api_instance.generic_request('/test', 'POST',
                                                                    body="this is a test",
                                                                    content_type='text/plain',
                                                                    accept='text/plain')
            self.assertTrue('HTTP response body' in str(context.exception))
            self.assertFalse('HTTP response headers' in str(context.exception))
            assert len(responses.calls) == 1
            assert responses.calls[0].request.url == '/test'
            assert responses.calls[0].request.host == 'localhost'
            assert responses.calls[0].request.scheme == 'http'

        run()
        assert_reset()


class TestFailedGenericRequestWithEnv(unittest.TestCase):

    def test_generic_request(self):
        """
        Given:
        - Request which should result in an ApiException
        When:
        - Environment variable DEMISTO_EXCEPTION_HEADER_LOGGING has been set to true
        Then:
        - Return ApiException with the headers in the error
        """
        import sys
        # Error should be the same in both Py2 and Py3, but Py2 does not support unittest mock in
        # the same way
        if sys.version_info[0] > 2:
            import os
            from demisto_client.demisto_api.rest import ApiException
            from unittest import mock

            @mock.patch.dict(os.environ, {"DEMISTO_EXCEPTION_HEADER_LOGGING": "true"})
            @responses.activate
            def run():
                responses.add('POST', '/test',
                              body="Not good",
                              status=400,
                              content_type='text/plain')
                api_instance = demisto_client.configure(base_url=host, api_key=api_key, debug=False)

                with self.assertRaises(ApiException) as context:
                    (_, _, _) = api_instance.generic_request('/test', 'POST',
                                                             body="this is a test",
                                                             content_type='text/plain',
                                                             accept='text/plain')
                self.assertTrue('HTTP response body' in str(context.exception))
                self.assertTrue('HTTP response headers' in str(context.exception))
                assert len(responses.calls) == 1
                assert responses.calls[0].request.url == '/test'
                assert responses.calls[0].request.host == 'localhost'
                assert responses.calls[0].request.scheme == 'http'
        else:
            def run():
                assert 1 == 1
        run()
        assert_reset()


class TestWithProxyBasicAuthentication(unittest.TestCase):

    def test_generic_request(self):
        """
        Given:
        - Request which use proxy authentication
        When:
        - Environment variable HTTP_PROXY and HTTPS_PROXY which contains a username and password with special characters
        Then:
        - Ensure the request made to the correct url via correct proxy username and password. Special character should be decoded.
        """
        import sys
        # Error should be the same in both Py2 and Py3, but Py2 does not support unittest mock in
        # the same way
        if sys.version_info[0] > 2:
            from unittest import mock

            @mock.patch.dict(os.environ, {"HTTP_PROXY": "http://user1:pass%21%23%24%25%5E@localhost:8080"})
            @mock.patch.dict(os.environ, {"HTTPS_PROXY": "http://user1:pass%21%23%24%25%5E@localhost:8080"})
            @responses.activate
            def run():
                responses.add('POST', 'http://localhost:8080/test',
                              body="Good",
                              status=200,
                              content_type='text/plain')
                api_instance = demisto_client.configure(base_url=host, api_key=api_key, debug=False)

                api_instance.generic_request('/test', 'POST',
                                            body="this is a test",
                                            content_type='text/plain',
                                            accept='text/plain')

                assert len(responses.calls) == 1
                assert responses.calls[0].request.url == 'http://localhost:8080/test'
                assert responses.calls[0].request.host == 'localhost'
                assert responses.calls[0].request.scheme == 'http'
                assert api_instance.api_client.configuration.proxy_auth == 'user1:pass!#$%^'
                
        else:
            def run():
                assert 1 == 1
        run()
        assert_reset()


def test_import_incidentfields(mocker):
    """
    Given:
        A path for a incidentfield.
    When:
        Importing incidentfields with partial errors.
    Then:
        Make sure the partial error is returned.
    """

    api_instance = demisto_client.configure(base_url=host, api_key=api_key, debug=False)

    raw_http_response = urllib3.response.HTTPResponse(body=b'{"incidentFields":[{"id":"evidence_description"}],'
                                                           b'"error":"Partial Error Description"}\n', status=200)
    mocker.patch.object(rest.RESTClientObject, 'POST', return_value=raw_http_response)
    with tempfile.NamedTemporaryFile() as tmp:
        res = api_instance.import_incident_fields(tmp.name)
    if isinstance(res, dict):
        assert res.get('error') == 'Partial Error Description'
    else:
        assert hasattr(res, 'error')
        assert res.error == 'Partial Error Description'


class TestConfigureClient:

    CONFIGURE_TEST_PARAMS = [
        (
            'api_key', 'username', {"header1": "value1", "header2": "value2"}, False
        ),
        (
            None, 'username', {"header1": "value1"}, True
        ),
        (
            'api_key', None, None, False
        ),
        (
            None, None, None, False
        ),
    ]

    CONFIGURE_TEST_PARAMS_ENV_VARIABLE = [
        (
            'api_key', 'username', "header1=value1,header2=value2", False
        ),
        (
            None, 'username', "header1=value1", True
        ),
        (
            'api_key', None, None, False
        ),
        (
            None, None, None, False
        ),
    ]

    @staticmethod
    def getenv_decorator(username=None, _api_key=None, additional_headers=None):
        def getenv_side_effect(parameter):
            if parameter == 'DEMISTO_API_KEY':
                return _api_key
            elif parameter == 'DEMISTO_USERNAME':
                return username
            elif parameter == 'DEMISTO_HTTP_HEADERS':
                return additional_headers
            return None

        return getenv_side_effect

    @staticmethod
    def configure_client(_api_key=None, username=None, additional_headers=None):
        if not (_api_key or os.getenv('DEMISTO_API_KEY')) and not (username or os.getenv('DEMISTO_USERNAME')):
            with pytest.raises(ValueError):
                demisto_client.configure(
                    base_url=host, api_key=_api_key, username=username, additional_headers=additional_headers
                )
        else:
            return demisto_client.configure(
                base_url=host, api_key=_api_key, username=username, additional_headers=additional_headers
            )

    def validate_additional_headers(
        self, api_instance, http_request_mocker, num_of_http_requests_mock, expected_additional_headers
    ):
        if not expected_additional_headers.items() <= api_instance.api_client.default_headers.items():
            return False

        self.run_http_request(api_instance, num_of_http_requests_mock=num_of_http_requests_mock)

        for i in range(len(http_request_mocker.call_args_list)):
            if not expected_additional_headers.items() <= http_request_mocker.call_args_list[i].kwargs['headers'].items():
                return False

        return True

    @staticmethod
    def run_http_request(api_instance, num_of_http_requests_mock):
        for i in range(num_of_http_requests_mock):
            api_instance.generic_request(path='/about', method='GET')

    @pytest.mark.parametrize(
        "_api_key, username, additional_headers, should_login_called",
        CONFIGURE_TEST_PARAMS
    )
    def test_configure_client_no_env_vars(self, mocker, _api_key, username, additional_headers, should_login_called):
        """
        Given:
            Case A: api key, username and additional headers were provided
            Case B: username and additional headers were provided
            Case C: only api key was provided
            Case D: neither of api key / username was provided

        When:
            configuring the client

        Then:
            Case A: make sure login was not called, authentication is via api key and
                    additional headers were sent in each request
            Case B: make sure login was called, authentication is via username/password and
                    additional headers were sent in each request
            Case C: make sure login was not called and authentication is via api key
            Case D: make sure an exception is raised
        """
        num_of_http_requests_mock = 5
        http_request_mocker = get_login_mocker(mocker, num_of_http_requests_mock=num_of_http_requests_mock)

        api_instance = self.configure_client(
            _api_key=_api_key, username=username, additional_headers=additional_headers
        )

        assert http_request_mocker.called == should_login_called

        if api_instance and additional_headers is not None:
            assert self.validate_additional_headers(
                api_instance,
                http_request_mocker,
                num_of_http_requests_mock=num_of_http_requests_mock,
                expected_additional_headers=additional_headers
            )

    @pytest.mark.parametrize(
        "_api_key, username, additional_headers, should_login_called",
        CONFIGURE_TEST_PARAMS_ENV_VARIABLE
    )
    def test_configure_client_env_vars(self, mocker, _api_key, username, additional_headers, should_login_called):
        """
        Given:
            Case A: api key, username and additional headers were provided through environment variables
            Case B: username and additional headers were provided through environment variables
            Case C: only api key was provided through environment variable
            Case D: neither of api key / username was provided at all

        When:
            configuring the client

        Then:
            Case A: make sure login was not called, authentication is via api key and
                    additional headers were sent in each request
            Case B: make sure login was called, authentication is via username/password, and
                    additional headers were sent in each request
            Case C: make sure login was not called and authentication is via api key
            Case D: make sure an exception is raised
        """
        num_of_http_requests_mock = 5

        http_request_mocker = get_login_mocker(mocker, num_of_http_requests_mock=num_of_http_requests_mock)
        mocker.patch.object(
            os,
            'getenv',
            side_effect=self.getenv_decorator(
                username=username, _api_key=_api_key, additional_headers=additional_headers
            )
        )

        api_instance = self.configure_client()
        assert http_request_mocker.called == should_login_called

        if api_instance and additional_headers is not None:
            expected_additional_headers = dict(header.split('=') for header in additional_headers.split(','))

            assert self.validate_additional_headers(
                api_instance,
                http_request_mocker,
                num_of_http_requests_mock=num_of_http_requests_mock,
                expected_additional_headers=expected_additional_headers
            )

    @pytest.mark.parametrize(
        "invalid_additional_headers",
        [
            "header1:value1,header2:value2",
            "header1,value1,header2,value2",
            "header1=value1,header2,value2",
            "header1=value1,header2value2",
            "header1=value1header2=value2",
            "test",
            "{header1:value1,header2:value2}",
            "test=test,",
            "test=test,test2=",
            "test=test, test2=a",
            "arg=arg,  c=d",
            "arg-arg,  c=d",
            "1=2,3=",
            "name=john,",
            "name=john age=42",
            "bar=",
            "name,",
            "header1=value1 header2=value2",
            "!=1 d",
            "!=1\nd"
        ]
    )
    def test_configure_client_invalid_additional_headers_form_env_var(self, mocker, invalid_additional_headers):
        """
        Given:
            invalid form of http headers

        When:
            configuring the client

        Then:
            make sure an exception is raised saying the format for invalid_additional_headers is invalid
        """
        mocker.patch.object(
            os,
            'getenv',
            side_effect=self.getenv_decorator(additional_headers=invalid_additional_headers)
        )

        with pytest.raises(ValueError) as exc:
            demisto_client.configure(base_url=host, api_key=api_key)

        assert exc.value.args[0] == f'{invalid_additional_headers} has invalid format, must be in the format ' \
                                    f'of header1=value1,header2=value2,...headerN=valueN'

    @pytest.mark.parametrize(
        "valid_additional_headers, expected_headers_as_dict",
        [
            (
                "header1=value1,header2=value2",
                {'header1': 'value1', 'header2': 'value2'}
            ),
            (
                "header1=value1",
                {'header1': 'value1'}
            ),
            (
                "blabla=1!,headers=2@",
                {'blabla': '1!', 'headers': '2@'}
            ),
            (
                "header1=with space,header2=another space",
                {'header1': 'with space', 'header2': 'another space'}
            ),
            (
                "header-1=value-1,header-2=value-2",
                {'header-1': 'value-1', 'header-2': 'value-2'}
            ),
            (
                "header1=header1,header2=header2,header3=header3,header4=header4,header5=header5",
                {'header1': 'header1', 'header2': 'header2', 'header3': 'header3', 'header4': 'header4', 'header5': 'header5'}
            ),
            (
                "PROXY=@$%#!djaj,PROXY2=[123]#%",
                {'PROXY': '@$%#!djaj', 'PROXY2': '[123]#%'}
            ),
            (
                "Proxy-Authorization=#$32dll1223--ds,header2=dkkfff",
                {'Proxy-Authorization': '#$32dll1223--ds', 'header2': 'dkkfff'}
            ),
            (
                "Content-type=123",
                {'Content-type': '123'}
            ),
            (
                "  ",
                {}
            ),
            (
                "",
                {}
            ),
        ]
    )
    def test_configure_client_valid_additional_headers_form_env_var(
        self, mocker, valid_additional_headers, expected_headers_as_dict
    ):
        """
        Given:
            valid form of http headers

        When:
            configuring the client

        Then:
            make sure the client is configured correctly, no exception is raised and
            headers were added to default headers for every http request
        """
        mocker.patch.object(
            os,
            'getenv',
            side_effect=self.getenv_decorator(additional_headers=valid_additional_headers)
        )

        client = demisto_client.configure(base_url=host, api_key=api_key)
        assert expected_headers_as_dict.items() <= client.api_client.default_headers.items()


# coding: utf-8

"""
    Demisto API

    This is the public REST API to integrate with the demisto server. HTTP request can be sent using any HTTP-client.  For an example dedicated client take a look at: https://github.com/demisto/demisto-py.  Requests must include API-key that can be generated in the Demisto web client under 'Settings' -> 'Integrations' -> 'API keys'   Optimistic Locking and Versioning\\:  When using Demisto REST API, you will need to make sure to work on the latest version of the item (incident, entry, etc.), otherwise, you will get a DB version error (which not allow you to override a newer item). In addition, you can pass 'version\\: -1' to force data override (make sure that other users data might be lost).  Assume that Alice and Bob both read the same data from Demisto server, then they both changed the data, and then both tried to write the new versions back to the server. Whose changes should be saved? Alice’s? Bob’s? To solve this, each data item in Demisto has a numeric incremental version. If Alice saved an item with version 4 and Bob trying to save the same item with version 3, Demisto will rollback Bob request and returns a DB version conflict error. Bob will need to get the latest item and work on it so Alice work will not get lost.  Example request using 'curl'\\:  ``` curl 'https://hostname:443/incidents/search' -H 'content-type: application/json' -H 'accept: application/json' -H 'Authorization: <API Key goes here>' --data-binary '{\"filter\":{\"query\":\"-status:closed -category:job\",\"period\":{\"by\":\"day\",\"fromValue\":7}}}' --compressed ```  # noqa: E501

    OpenAPI spec version: 2.0.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six

from demisto_client.demisto_api.models.date_range_filter import DateRangeFilter  # noqa: F401,E501
from demisto_client.demisto_api.models.duration import Duration  # noqa: F401,E501
from demisto_client.demisto_api.models.order import Order  # noqa: F401,E501
from demisto_client.demisto_api.models.period import Period  # noqa: F401,E501


class IndicatorFilter(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'cache': 'dict(str, list[str])',
        'early_time_in_page': 'datetime',
        'first_seen': 'DateRangeFilter',
        'from_date': 'datetime',
        'from_date_license': 'datetime',
        'last_seen': 'DateRangeFilter',
        'later_time_in_page': 'datetime',
        'page': 'int',
        'period': 'Period',
        'prev_page': 'bool',
        'query': 'str',
        'search_after': 'list[str]',
        'search_before': 'list[str]',
        'size': 'int',
        'sort': 'list[Order]',
        'time_frame': 'Duration',
        'to_date': 'datetime'
    }

    attribute_map = {
        'cache': 'Cache',
        'early_time_in_page': 'earlyTimeInPage',
        'first_seen': 'firstSeen',
        'from_date': 'fromDate',
        'from_date_license': 'fromDateLicense',
        'last_seen': 'lastSeen',
        'later_time_in_page': 'laterTimeInPage',
        'page': 'page',
        'period': 'period',
        'prev_page': 'prevPage',
        'query': 'query',
        'search_after': 'searchAfter',
        'search_before': 'searchBefore',
        'size': 'size',
        'sort': 'sort',
        'time_frame': 'timeFrame',
        'to_date': 'toDate'
    }

    def __init__(self, cache=None, early_time_in_page=None, first_seen=None, from_date=None, from_date_license=None, last_seen=None, later_time_in_page=None, page=None, period=None, prev_page=None, query=None, search_after=None, search_before=None, size=None, sort=None, time_frame=None, to_date=None):  # noqa: E501
        """IndicatorFilter - a model defined in Swagger"""  # noqa: E501

        self._cache = None
        self._early_time_in_page = None
        self._first_seen = None
        self._from_date = None
        self._from_date_license = None
        self._last_seen = None
        self._later_time_in_page = None
        self._page = None
        self._period = None
        self._prev_page = None
        self._query = None
        self._search_after = None
        self._search_before = None
        self._size = None
        self._sort = None
        self._time_frame = None
        self._to_date = None
        self.discriminator = None

        if cache is not None:
            self.cache = cache
        if early_time_in_page is not None:
            self.early_time_in_page = early_time_in_page
        if first_seen is not None:
            self.first_seen = first_seen
        if from_date is not None:
            self.from_date = from_date
        if from_date_license is not None:
            self.from_date_license = from_date_license
        if last_seen is not None:
            self.last_seen = last_seen
        if later_time_in_page is not None:
            self.later_time_in_page = later_time_in_page
        if page is not None:
            self.page = page
        if period is not None:
            self.period = period
        if prev_page is not None:
            self.prev_page = prev_page
        if query is not None:
            self.query = query
        if search_after is not None:
            self.search_after = search_after
        if search_before is not None:
            self.search_before = search_before
        if size is not None:
            self.size = size
        if sort is not None:
            self.sort = sort
        if time_frame is not None:
            self.time_frame = time_frame
        if to_date is not None:
            self.to_date = to_date

    @property
    def cache(self):
        """Gets the cache of this IndicatorFilter.  # noqa: E501

        Cache of join functions  # noqa: E501

        :return: The cache of this IndicatorFilter.  # noqa: E501
        :rtype: dict(str, list[str])
        """
        return self._cache

    @cache.setter
    def cache(self, cache):
        """Sets the cache of this IndicatorFilter.

        Cache of join functions  # noqa: E501

        :param cache: The cache of this IndicatorFilter.  # noqa: E501
        :type: dict(str, list[str])
        """

        self._cache = cache

    @property
    def early_time_in_page(self):
        """Gets the early_time_in_page of this IndicatorFilter.  # noqa: E501


        :return: The early_time_in_page of this IndicatorFilter.  # noqa: E501
        :rtype: datetime
        """
        return self._early_time_in_page

    @early_time_in_page.setter
    def early_time_in_page(self, early_time_in_page):
        """Sets the early_time_in_page of this IndicatorFilter.


        :param early_time_in_page: The early_time_in_page of this IndicatorFilter.  # noqa: E501
        :type: datetime
        """

        self._early_time_in_page = early_time_in_page

    @property
    def first_seen(self):
        """Gets the first_seen of this IndicatorFilter.  # noqa: E501


        :return: The first_seen of this IndicatorFilter.  # noqa: E501
        :rtype: DateRangeFilter
        """
        return self._first_seen

    @first_seen.setter
    def first_seen(self, first_seen):
        """Sets the first_seen of this IndicatorFilter.


        :param first_seen: The first_seen of this IndicatorFilter.  # noqa: E501
        :type: DateRangeFilter
        """

        self._first_seen = first_seen

    @property
    def from_date(self):
        """Gets the from_date of this IndicatorFilter.  # noqa: E501


        :return: The from_date of this IndicatorFilter.  # noqa: E501
        :rtype: datetime
        """
        return self._from_date

    @from_date.setter
    def from_date(self, from_date):
        """Sets the from_date of this IndicatorFilter.


        :param from_date: The from_date of this IndicatorFilter.  # noqa: E501
        :type: datetime
        """

        self._from_date = from_date

    @property
    def from_date_license(self):
        """Gets the from_date_license of this IndicatorFilter.  # noqa: E501


        :return: The from_date_license of this IndicatorFilter.  # noqa: E501
        :rtype: datetime
        """
        return self._from_date_license

    @from_date_license.setter
    def from_date_license(self, from_date_license):
        """Sets the from_date_license of this IndicatorFilter.


        :param from_date_license: The from_date_license of this IndicatorFilter.  # noqa: E501
        :type: datetime
        """

        self._from_date_license = from_date_license

    @property
    def last_seen(self):
        """Gets the last_seen of this IndicatorFilter.  # noqa: E501


        :return: The last_seen of this IndicatorFilter.  # noqa: E501
        :rtype: DateRangeFilter
        """
        return self._last_seen

    @last_seen.setter
    def last_seen(self, last_seen):
        """Sets the last_seen of this IndicatorFilter.


        :param last_seen: The last_seen of this IndicatorFilter.  # noqa: E501
        :type: DateRangeFilter
        """

        self._last_seen = last_seen

    @property
    def later_time_in_page(self):
        """Gets the later_time_in_page of this IndicatorFilter.  # noqa: E501


        :return: The later_time_in_page of this IndicatorFilter.  # noqa: E501
        :rtype: datetime
        """
        return self._later_time_in_page

    @later_time_in_page.setter
    def later_time_in_page(self, later_time_in_page):
        """Sets the later_time_in_page of this IndicatorFilter.


        :param later_time_in_page: The later_time_in_page of this IndicatorFilter.  # noqa: E501
        :type: datetime
        """

        self._later_time_in_page = later_time_in_page

    @property
    def page(self):
        """Gets the page of this IndicatorFilter.  # noqa: E501

        0-based page  # noqa: E501

        :return: The page of this IndicatorFilter.  # noqa: E501
        :rtype: int
        """
        return self._page

    @page.setter
    def page(self, page):
        """Sets the page of this IndicatorFilter.

        0-based page  # noqa: E501

        :param page: The page of this IndicatorFilter.  # noqa: E501
        :type: int
        """

        self._page = page

    @property
    def period(self):
        """Gets the period of this IndicatorFilter.  # noqa: E501


        :return: The period of this IndicatorFilter.  # noqa: E501
        :rtype: Period
        """
        return self._period

    @period.setter
    def period(self, period):
        """Sets the period of this IndicatorFilter.


        :param period: The period of this IndicatorFilter.  # noqa: E501
        :type: Period
        """

        self._period = period

    @property
    def prev_page(self):
        """Gets the prev_page of this IndicatorFilter.  # noqa: E501

        MT support - these fields are for indicator search according to calculatedTime  # noqa: E501

        :return: The prev_page of this IndicatorFilter.  # noqa: E501
        :rtype: bool
        """
        return self._prev_page

    @prev_page.setter
    def prev_page(self, prev_page):
        """Sets the prev_page of this IndicatorFilter.

        MT support - these fields are for indicator search according to calculatedTime  # noqa: E501

        :param prev_page: The prev_page of this IndicatorFilter.  # noqa: E501
        :type: bool
        """

        self._prev_page = prev_page

    @property
    def query(self):
        """Gets the query of this IndicatorFilter.  # noqa: E501


        :return: The query of this IndicatorFilter.  # noqa: E501
        :rtype: str
        """
        return self._query

    @query.setter
    def query(self, query):
        """Sets the query of this IndicatorFilter.


        :param query: The query of this IndicatorFilter.  # noqa: E501
        :type: str
        """

        self._query = query

    @property
    def search_after(self):
        """Gets the search_after of this IndicatorFilter.  # noqa: E501

        Efficient next page, pass max sort value from previous page  # noqa: E501

        :return: The search_after of this IndicatorFilter.  # noqa: E501
        :rtype: list[str]
        """
        return self._search_after

    @search_after.setter
    def search_after(self, search_after):
        """Sets the search_after of this IndicatorFilter.

        Efficient next page, pass max sort value from previous page  # noqa: E501

        :param search_after: The search_after of this IndicatorFilter.  # noqa: E501
        :type: list[str]
        """

        self._search_after = search_after

    @property
    def search_before(self):
        """Gets the search_before of this IndicatorFilter.  # noqa: E501

        Efficient prev page, pass min sort value from next page  # noqa: E501

        :return: The search_before of this IndicatorFilter.  # noqa: E501
        :rtype: list[str]
        """
        return self._search_before

    @search_before.setter
    def search_before(self, search_before):
        """Sets the search_before of this IndicatorFilter.

        Efficient prev page, pass min sort value from next page  # noqa: E501

        :param search_before: The search_before of this IndicatorFilter.  # noqa: E501
        :type: list[str]
        """

        self._search_before = search_before

    @property
    def size(self):
        """Gets the size of this IndicatorFilter.  # noqa: E501

        Size is limited to 1000, if not passed it defaults to 0, and no results will return  # noqa: E501

        :return: The size of this IndicatorFilter.  # noqa: E501
        :rtype: int
        """
        return self._size

    @size.setter
    def size(self, size):
        """Sets the size of this IndicatorFilter.

        Size is limited to 1000, if not passed it defaults to 0, and no results will return  # noqa: E501

        :param size: The size of this IndicatorFilter.  # noqa: E501
        :type: int
        """

        self._size = size

    @property
    def sort(self):
        """Gets the sort of this IndicatorFilter.  # noqa: E501

        The sort order  # noqa: E501

        :return: The sort of this IndicatorFilter.  # noqa: E501
        :rtype: list[Order]
        """
        return self._sort

    @sort.setter
    def sort(self, sort):
        """Sets the sort of this IndicatorFilter.

        The sort order  # noqa: E501

        :param sort: The sort of this IndicatorFilter.  # noqa: E501
        :type: list[Order]
        """

        self._sort = sort

    @property
    def time_frame(self):
        """Gets the time_frame of this IndicatorFilter.  # noqa: E501


        :return: The time_frame of this IndicatorFilter.  # noqa: E501
        :rtype: Duration
        """
        return self._time_frame

    @time_frame.setter
    def time_frame(self, time_frame):
        """Sets the time_frame of this IndicatorFilter.


        :param time_frame: The time_frame of this IndicatorFilter.  # noqa: E501
        :type: Duration
        """

        self._time_frame = time_frame

    @property
    def to_date(self):
        """Gets the to_date of this IndicatorFilter.  # noqa: E501


        :return: The to_date of this IndicatorFilter.  # noqa: E501
        :rtype: datetime
        """
        return self._to_date

    @to_date.setter
    def to_date(self, to_date):
        """Sets the to_date of this IndicatorFilter.


        :param to_date: The to_date of this IndicatorFilter.  # noqa: E501
        :type: datetime
        """

        self._to_date = to_date

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(IndicatorFilter, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, IndicatorFilter):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Demisto API

    This is the public REST API to integrate with the demisto server. HTTP request can be sent using any HTTP-client.  For an example dedicated client take a look at: https://github.com/demisto/demisto-py.  Requests must include API-key that can be generated in the Demisto web client under 'Settings' -> 'Integrations' -> 'API keys'   Optimistic Locking and Versioning\\:  When using Demisto REST API, you will need to make sure to work on the latest version of the item (incident, entry, etc.), otherwise, you will get a DB version error (which not allow you to override a newer item). In addition, you can pass 'version\\: -1' to force data override (make sure that other users data might be lost).  Assume that Alice and Bob both read the same data from Demisto server, then they both changed the data, and then both tried to write the new versions back to the server. Whose changes should be saved? Alice’s? Bob’s? To solve this, each data item in Demisto has a numeric incremental version. If Alice saved an item with version 4 and Bob trying to save the same item with version 3, Demisto will rollback Bob request and returns a DB version conflict error. Bob will need to get the latest item and work on it so Alice work will not get lost.  Example request using 'curl'\\:  ``` curl 'https://hostname:443/incidents/search' -H 'content-type: application/json' -H 'accept: application/json' -H 'Authorization: <API Key goes here>' --data-binary '{\"filter\":{\"query\":\"-status:closed -category:job\",\"period\":{\"by\":\"day\",\"fromValue\":7}}}' --compressed ```  # noqa: E501

    OpenAPI spec version: 2.0.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six

from demisto_client.demisto_api.models.config_data_type import ConfigDataType  # noqa: F401,E501


class ConfigField(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'default_value': 'str',
        'display': 'str',
        'hidden': 'bool',
        'info': 'str',
        'name': 'str',
        'options': 'list[str]',
        'required': 'bool',
        'type': 'ConfigDataType'
    }

    attribute_map = {
        'default_value': 'defaultValue',
        'display': 'display',
        'hidden': 'hidden',
        'info': 'info',
        'name': 'name',
        'options': 'options',
        'required': 'required',
        'type': 'type'
    }

    def __init__(self, default_value=None, display=None, hidden=None, info=None, name=None, options=None, required=None, type=None):  # noqa: E501
        """ConfigField - a model defined in Swagger"""  # noqa: E501

        self._default_value = None
        self._display = None
        self._hidden = None
        self._info = None
        self._name = None
        self._options = None
        self._required = None
        self._type = None
        self.discriminator = None

        if default_value is not None:
            self.default_value = default_value
        if display is not None:
            self.display = display
        if hidden is not None:
            self.hidden = hidden
        if info is not None:
            self.info = info
        if name is not None:
            self.name = name
        if options is not None:
            self.options = options
        if required is not None:
            self.required = required
        if type is not None:
            self.type = type

    @property
    def default_value(self):
        """Gets the default_value of this ConfigField.  # noqa: E501


        :return: The default_value of this ConfigField.  # noqa: E501
        :rtype: str
        """
        return self._default_value

    @default_value.setter
    def default_value(self, default_value):
        """Sets the default_value of this ConfigField.


        :param default_value: The default_value of this ConfigField.  # noqa: E501
        :type: str
        """

        self._default_value = default_value

    @property
    def display(self):
        """Gets the display of this ConfigField.  # noqa: E501


        :return: The display of this ConfigField.  # noqa: E501
        :rtype: str
        """
        return self._display

    @display.setter
    def display(self, display):
        """Sets the display of this ConfigField.


        :param display: The display of this ConfigField.  # noqa: E501
        :type: str
        """

        self._display = display

    @property
    def hidden(self):
        """Gets the hidden of this ConfigField.  # noqa: E501


        :return: The hidden of this ConfigField.  # noqa: E501
        :rtype: bool
        """
        return self._hidden

    @hidden.setter
    def hidden(self, hidden):
        """Sets the hidden of this ConfigField.


        :param hidden: The hidden of this ConfigField.  # noqa: E501
        :type: bool
        """

        self._hidden = hidden

    @property
    def info(self):
        """Gets the info of this ConfigField.  # noqa: E501


        :return: The info of this ConfigField.  # noqa: E501
        :rtype: str
        """
        return self._info

    @info.setter
    def info(self, info):
        """Sets the info of this ConfigField.


        :param info: The info of this ConfigField.  # noqa: E501
        :type: str
        """

        self._info = info

    @property
    def name(self):
        """Gets the name of this ConfigField.  # noqa: E501


        :return: The name of this ConfigField.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this ConfigField.


        :param name: The name of this ConfigField.  # noqa: E501
        :type: str
        """

        self._name = name

    @property
    def options(self):
        """Gets the options of this ConfigField.  # noqa: E501


        :return: The options of this ConfigField.  # noqa: E501
        :rtype: list[str]
        """
        return self._options

    @options.setter
    def options(self, options):
        """Sets the options of this ConfigField.


        :param options: The options of this ConfigField.  # noqa: E501
        :type: list[str]
        """

        self._options = options

    @property
    def required(self):
        """Gets the required of this ConfigField.  # noqa: E501


        :return: The required of this ConfigField.  # noqa: E501
        :rtype: bool
        """
        return self._required

    @required.setter
    def required(self, required):
        """Sets the required of this ConfigField.


        :param required: The required of this ConfigField.  # noqa: E501
        :type: bool
        """

        self._required = required

    @property
    def type(self):
        """Gets the type of this ConfigField.  # noqa: E501


        :return: The type of this ConfigField.  # noqa: E501
        :rtype: ConfigDataType
        """
        return self._type

    @type.setter
    def type(self, type):
        """Sets the type of this ConfigField.


        :param type: The type of this ConfigField.  # noqa: E501
        :type: ConfigDataType
        """

        self._type = type

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(ConfigField, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ConfigField):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Demisto API

    This is the public REST API to integrate with the demisto server. HTTP request can be sent using any HTTP-client.  For an example dedicated client take a look at: https://github.com/demisto/demisto-py.  Requests must include API-key that can be generated in the Demisto web client under 'Settings' -> 'Integrations' -> 'API keys'   Optimistic Locking and Versioning\\:  When using Demisto REST API, you will need to make sure to work on the latest version of the item (incident, entry, etc.), otherwise, you will get a DB version error (which not allow you to override a newer item). In addition, you can pass 'version\\: -1' to force data override (make sure that other users data might be lost).  Assume that Alice and Bob both read the same data from Demisto server, then they both changed the data, and then both tried to write the new versions back to the server. Whose changes should be saved? Alice’s? Bob’s? To solve this, each data item in Demisto has a numeric incremental version. If Alice saved an item with version 4 and Bob trying to save the same item with version 3, Demisto will rollback Bob request and returns a DB version conflict error. Bob will need to get the latest item and work on it so Alice work will not get lost.  Example request using 'curl'\\:  ``` curl 'https://hostname:443/incidents/search' -H 'content-type: application/json' -H 'accept: application/json' -H 'Authorization: <API Key goes here>' --data-binary '{\"filter\":{\"query\":\"-status:closed -category:job\",\"period\":{\"by\":\"day\",\"fromValue\":7}}}' --compressed ```  # noqa: E501

    OpenAPI spec version: 2.0.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six

from demisto_client.demisto_api.models.expiration_policy import ExpirationPolicy  # noqa: F401,E501
from demisto_client.demisto_api.models.expiration_settings_source import ExpirationSettingsSource  # noqa: F401,E501


class ExpirationSource(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'brand': 'str',
        'expiration_interval': 'int',
        'expiration_policy': 'ExpirationPolicy',
        'instance': 'str',
        'module_id': 'str',
        'set_time': 'datetime',
        'source': 'ExpirationSettingsSource',
        'user': 'str'
    }

    attribute_map = {
        'brand': 'brand',
        'expiration_interval': 'expirationInterval',
        'expiration_policy': 'expirationPolicy',
        'instance': 'instance',
        'module_id': 'moduleId',
        'set_time': 'setTime',
        'source': 'source',
        'user': 'user'
    }

    def __init__(self, brand=None, expiration_interval=None, expiration_policy=None, instance=None, module_id=None, set_time=None, source=None, user=None):  # noqa: E501
        """ExpirationSource - a model defined in Swagger"""  # noqa: E501

        self._brand = None
        self._expiration_interval = None
        self._expiration_policy = None
        self._instance = None
        self._module_id = None
        self._set_time = None
        self._source = None
        self._user = None
        self.discriminator = None

        if brand is not None:
            self.brand = brand
        if expiration_interval is not None:
            self.expiration_interval = expiration_interval
        if expiration_policy is not None:
            self.expiration_policy = expiration_policy
        if instance is not None:
            self.instance = instance
        if module_id is not None:
            self.module_id = module_id
        if set_time is not None:
            self.set_time = set_time
        if source is not None:
            self.source = source
        if user is not None:
            self.user = user

    @property
    def brand(self):
        """Gets the brand of this ExpirationSource.  # noqa: E501


        :return: The brand of this ExpirationSource.  # noqa: E501
        :rtype: str
        """
        return self._brand

    @brand.setter
    def brand(self, brand):
        """Sets the brand of this ExpirationSource.


        :param brand: The brand of this ExpirationSource.  # noqa: E501
        :type: str
        """

        self._brand = brand

    @property
    def expiration_interval(self):
        """Gets the expiration_interval of this ExpirationSource.  # noqa: E501


        :return: The expiration_interval of this ExpirationSource.  # noqa: E501
        :rtype: int
        """
        return self._expiration_interval

    @expiration_interval.setter
    def expiration_interval(self, expiration_interval):
        """Sets the expiration_interval of this ExpirationSource.


        :param expiration_interval: The expiration_interval of this ExpirationSource.  # noqa: E501
        :type: int
        """

        self._expiration_interval = expiration_interval

    @property
    def expiration_policy(self):
        """Gets the expiration_policy of this ExpirationSource.  # noqa: E501


        :return: The expiration_policy of this ExpirationSource.  # noqa: E501
        :rtype: ExpirationPolicy
        """
        return self._expiration_policy

    @expiration_policy.setter
    def expiration_policy(self, expiration_policy):
        """Sets the expiration_policy of this ExpirationSource.


        :param expiration_policy: The expiration_policy of this ExpirationSource.  # noqa: E501
        :type: ExpirationPolicy
        """

        self._expiration_policy = expiration_policy

    @property
    def instance(self):
        """Gets the instance of this ExpirationSource.  # noqa: E501


        :return: The instance of this ExpirationSource.  # noqa: E501
        :rtype: str
        """
        return self._instance

    @instance.setter
    def instance(self, instance):
        """Sets the instance of this ExpirationSource.


        :param instance: The instance of this ExpirationSource.  # noqa: E501
        :type: str
        """

        self._instance = instance

    @property
    def module_id(self):
        """Gets the module_id of this ExpirationSource.  # noqa: E501


        :return: The module_id of this ExpirationSource.  # noqa: E501
        :rtype: str
        """
        return self._module_id

    @module_id.setter
    def module_id(self, module_id):
        """Sets the module_id of this ExpirationSource.


        :param module_id: The module_id of this ExpirationSource.  # noqa: E501
        :type: str
        """

        self._module_id = module_id

    @property
    def set_time(self):
        """Gets the set_time of this ExpirationSource.  # noqa: E501


        :return: The set_time of this ExpirationSource.  # noqa: E501
        :rtype: datetime
        """
        return self._set_time

    @set_time.setter
    def set_time(self, set_time):
        """Sets the set_time of this ExpirationSource.


        :param set_time: The set_time of this ExpirationSource.  # noqa: E501
        :type: datetime
        """

        self._set_time = set_time

    @property
    def source(self):
        """Gets the source of this ExpirationSource.  # noqa: E501


        :return: The source of this ExpirationSource.  # noqa: E501
        :rtype: ExpirationSettingsSource
        """
        return self._source

    @source.setter
    def source(self, source):
        """Sets the source of this ExpirationSource.


        :param source: The source of this ExpirationSource.  # noqa: E501
        :type: ExpirationSettingsSource
        """

        self._source = source

    @property
    def user(self):
        """Gets the user of this ExpirationSource.  # noqa: E501


        :return: The user of this ExpirationSource.  # noqa: E501
        :rtype: str
        """
        return self._user

    @user.setter
    def user(self, user):
        """Sets the user of this ExpirationSource.


        :param user: The user of this ExpirationSource.  # noqa: E501
        :type: str
        """

        self._user = user

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(ExpirationSource, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ExpirationSource):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



import re

import demisto_client.demisto_api as demisto_api
import six
import os
import datetime
import tzlocal
import json

from demisto_client.demisto_api import ApiClient
from demisto_client.demisto_api.configuration import Configuration
from pkg_resources import get_distribution, DistributionNotFound
from distutils.version import LooseVersion

try:
    __version__ = get_distribution(__name__).version
except DistributionNotFound:
    # package is not installed
    __version__ = 'dev'


DEMISTO_HTTP_HEADERS_REGEX_PATTERN = r'^([\w-]+=[^=,\n]+)(,[\w-]+=[^=,\n]+)*$'


def configure(base_url=None, api_key=None, advanced_api_key=None, verify_ssl=None, proxy=None, username=None, password=None,
              ssl_ca_cert=None, debug=False, connection_pool_maxsize=None, auth_id=None, additional_headers=None):
    """
    This wrapper provides an easier to use method of configuring the API client. The base
    Configuration method is still exposed if you wish to further configure the API Client.

    To avoid hard coding configurations in your code, it is possible to specify configuration params
    as the following environment variables (env variables will be used if parameters are not specified):

    * DEMISTO_BASE_URL
    * DEMISTO_API_KEY
    * DEMISTO_ADVANCED_API_KEY
    * DEMISTO_USERNAME
    * DEMISTO_PASSWORD
    * DEMISTO_VERIFY_SSL (true/false. Default: true)
    * DEMISTO_HTTP_HEADERS (must be in the form of: header1=value1,header2=value2,header3=value3,...headerN=valueN)
    * DEMISTO_API_KEY_ID
    * SSL_CERT_FILE (specify an alternate certificate bundle)
    * DEMISTO_CONNECTION_POOL_MAXSIZE (specify a connection pool max size)
    * HTTP_PROXY or HTTPS_PROXY (If you are using a proxy)

    :param base_url: str - Base url of your Demisto instance.
    :param api_key: str - API key generated by your instance.
    :param advanced_api_key: str - Advanced API key generated by your instance.
    :param username: str - Username of the user account.
    :param password: str - Password of the user account.
    :param verify_ssl: bool - Indicates if valid SSLs are required for connection. If not specified (None)
        will default to True.
    :param proxy: str - The URL of the proxy to be used.
    :param ssl_ca_cert: str - specify an alternate certificate bundle
    :param debug: bool - Include verbose logging.
    :param connection_pool_maxsize: int - specify a connection max pool size
    :param auth_id: str - api_key_id only for the xsiam
    :param additional_headers: dict - any additional headers to send to every http request to demisto.
    :return: Returns an API client configuration identical to the Configuration() method.
    """
    base_url = base_url or os.getenv('DEMISTO_BASE_URL')
    if not base_url:
        raise RuntimeError('the DEMISTO_BASE_URL value is not set.')
    api_key = api_key or os.getenv('DEMISTO_API_KEY')
    auth_id = auth_id or os.getenv('DEMISTO_API_KEY_ID') or os.getenv('XSIAM_AUTH_ID')
    username = username or os.getenv('DEMISTO_USERNAME')
    password = password or os.getenv('DEMISTO_PASSWORD')
    advanced_api_key = advanced_api_key or os.getenv('DEMISTO_ADVANCED_API_KEY')

    if additional_headers is None:
        if headers := os.getenv('DEMISTO_HTTP_HEADERS'):
            if re.match(r'^ *$', headers):  # catch any case of empty string
                additional_headers = {}
            elif re.match(DEMISTO_HTTP_HEADERS_REGEX_PATTERN, headers):
                additional_headers = dict(header.strip().split('=') for header in headers.split(','))
            else:
                raise ValueError(
                    f'{headers} has invalid format, must be in the format of header1=value1,header2=value2,...headerN=valueN'
                )
        else:
            additional_headers = {}
    if ssl_ca_cert is None:
        ssl_ca_cert = os.getenv('SSL_CERT_FILE')
    if verify_ssl is None:
        verify_env = os.getenv('DEMISTO_VERIFY_SSL')
        if verify_env:
            verify_ssl = verify_env.lower() not in ['false', '0', 'no']
        else:
            verify_ssl = True
    if proxy is None:
        if base_url.lower().startswith('http'):
            proxy = os.getenv('HTTP_PROXY')
        elif base_url.lower().startswith('https'):
            proxy = os.getenv('HTTPS_PROXY')
    if connection_pool_maxsize is None:
        connection_pool_maxsize = os.getenv('DEMISTO_CONNECTION_POOL_MAXSIZE')
        if connection_pool_maxsize:
            if not connection_pool_maxsize.isdigit():
                err_msg = ('DEMISTO_CONNECTION_POOL_MAXSIZE env variable should be set to a number'
                           f' but instead received "{connection_pool_maxsize}"')
                raise ValueError(err_msg)
            else:
                connection_pool_maxsize = int(connection_pool_maxsize)
    if not base_url:
        raise ValueError('You must specify base_url either as a parameter or via env variable: DEMISTO_BASE_URL')
    if not api_key and not username and not advanced_api_key:
        raise ValueError('You must specify either api_key or advanced_api_key or username/password either as parameters or use env variables:\n'
                         '* DEMISTO_API_KEY\n'
                         '* DEMISTO_ADVANCED_API_KEY\n'
                         '* DEMISTO_USERNAME\n'
                         '* DEMISTO_PASSWORD'
                         )
    if auth_id and not (api_key or advanced_api_key):
        raise ValueError('You must specify either api_key or advanced_api_key or use env variable DEMISTO_API_KEY or DEMISTO_ADVANCED_API_KEY ' 
                         'to use Cortex XSIAM api or XSOAR 8+, or remove the auth_id and/or the env variable DEMISTO_API_KEY_ID '
                         'to use Cortex XSOAR api:\n')
    configuration = Configuration()
    if advanced_api_key:
        setattr(configuration, 'auth_signed_key', advanced_api_key)
    else:
        configuration.api_key['Authorization'] = api_key
    configuration.host = os.path.join(base_url)
    if auth_id:
        configuration.api_key['x-xdr-auth-id'] = auth_id
        configuration.host = os.path.join(configuration.host, 'xsoar')
        if not configuration.host.startswith("https://api-"):
            configuration.host = configuration.host.replace('https://', 'https://api-')
    configuration.verify_ssl = verify_ssl
    configuration.proxy = proxy
    configuration.debug = debug
    configuration.ssl_ca_cert = ssl_ca_cert
    if connection_pool_maxsize:
        configuration.connection_pool_maxsize = connection_pool_maxsize

    if api_key or auth_id:
        api_client = ApiClient(configuration)
        api_client.user_agent = 'demisto-py/' + __version__
        api_client.default_headers.update(additional_headers)
        api_instance = demisto_api.DefaultApi(api_client)
        return api_instance
    else:
        api_instance = login(base_url=base_url, username=username, password=password,
                             verify_ssl=verify_ssl, proxy=proxy, debug=debug, additional_headers=additional_headers)
        return api_instance


def login(
    base_url=None, username=None, password=None, verify_ssl=True, proxy=None, debug=False, additional_headers=None
):
    additional_headers = additional_headers or {}
    configuration_orig = Configuration()
    configuration_orig.host = base_url or os.getenv('DEMISTO_BASE_URL', None)
    if isinstance(configuration_orig.host, str):
        configuration_orig.host = configuration_orig.host.rstrip('/')
    configuration_orig.verify_ssl = verify_ssl
    configuration_orig.proxy = proxy
    configuration_orig.debug = debug
    connection_pool_maxsize = os.getenv('DEMISTO_CONNECTION_POOL_MAXSIZE')
    if connection_pool_maxsize:
        if connection_pool_maxsize.isdigit():
            connection_pool_maxsize = int(connection_pool_maxsize)
            configuration_orig.connection_pool_maxsize = connection_pool_maxsize
        else:
            err_msg = ('DEMISTO_CONNECTION_POOL_MAXSIZE env variable should be set to a number'
                       f' but instead received "{connection_pool_maxsize}"')
            raise ValueError(err_msg)
    api_client = ApiClient(configuration_orig)
    api_client.user_agent = 'demisto-py/' + __version__
    api_client.default_headers.update(additional_headers)
    api_instance = demisto_api.DefaultApi(api_client)
    body = {
        "user": username,
        "password": password
    }
    res = generic_request_func(self=api_instance, path='/', method='GET', body=body,
                               accept='application/json', content_type='application/json')
    cookies = res[2]['Set-Cookie']
    cookie_jar = cookies.split(';')
    xsrf_token_raw = cookie_jar[0]
    xsrf_token = xsrf_token_raw.replace('XSRF-TOKEN=', '')
    configuration = Configuration()
    configuration.host = base_url or os.getenv('DEMISTO_BASE_URL', None)
    if isinstance(configuration.host, str):
        configuration.host = configuration.host.rstrip('/')
    configuration.verify_ssl = verify_ssl
    configuration.proxy = proxy
    configuration.debug = debug
    if connection_pool_maxsize and isinstance(connection_pool_maxsize, int):
        configuration.connection_pool_maxsize = connection_pool_maxsize
    api_client = ApiClient(configuration, header_name="X-XSRF-TOKEN", header_value=xsrf_token,
                           cookie=cookies)
    api_client.user_agent = 'demisto-py/' + __version__
    api_client.default_headers.update(additional_headers)
    mid_client = demisto_api.DefaultApi(api_client)
    second_call = generic_request_func(self=mid_client, path='/login', method='POST', body=body,
                                       accept='application/json', content_type='application/json')
    updated_cookies = cookies + '; ' + second_call[2]['Set-Cookie']
    mid_api_client = ApiClient(configuration, header_name="X-XSRF-TOKEN", header_value=xsrf_token,
                               cookie=updated_cookies)
    mid_api_client.user_agent = 'demisto-py/' + __version__
    mid_api_client.default_headers.update(additional_headers)
    final_api_client = demisto_api.DefaultApi(mid_api_client)

    return final_api_client


def to_extended_dict(o):
    """
    In some cases, the models do not use the attribute map which causes the server to not return
    valid results. This function checks to see if an attribute map is part of the object and if so,
    will apply the changes to request object. If not, it will default to use the models as defined
    swagger.

    :param o: Request object.
    :return: Formatted request object.
    """
    result = {}

    if hasattr(o, "attribute_map"):
        o_map = o.attribute_map

        for attr, _ in six.iteritems(o.swagger_types):
            value = getattr(o, attr)
            if isinstance(value, list):
                result[o_map[attr]] = list(map(
                    lambda x: to_extended_dict(x) if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[o_map[attr]] = to_extended_dict(value)
            elif isinstance(value, dict):
                result[o_map[attr]] = dict(map(
                    lambda item: (item[0], to_extended_dict(item[1]))
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            elif isinstance(value, datetime.datetime):
                if not value.tzinfo:  # no tz defined -> use machine local
                    local_tz = tzlocal.get_localzone()
                    value = value.replace(tzinfo=local_tz)
                result[o_map[attr]] = value.isoformat()
            else:
                result[o_map[attr]] = value
    else:
        o.to_dict() if hasattr(o, "to_dict") else o
        result = o
    return result


def generic_request_func(self, path, method, body=None, **kwargs):
    """
    This method allows for generic requests to be made against the Demisto server to endpoints that
    may not be exposed directly.
    :param path: str - Path of the endpoint.
    :param method: str - Type of API method to use.
    :param body: dict - Dict object of request body.
    :param content_type: str - optional content type of body.
    :param accept: str/list[str] - optional content type to accept.
    :param response_type: str - optional response type to return. Default: 'str'. To get an object specify 'object'.

    :return: tuple of (response_data, response_code, headers).
    """
    if not path.startswith('/'):
        path = '/' + path

    all_params = ['']  # noqa: E501
    all_params.append('async_req')
    all_params.append('_return_http_data_only')
    all_params.append('_preload_content')
    all_params.append('_request_timeout')
    all_params.append('content_type')
    all_params.append('accept')
    all_params.append('response_type')

    params = locals()
    for key, val in six.iteritems(params['kwargs']):
        if key not in all_params:
            raise TypeError(
                "Got an unexpected keyword argument '%s'"
                " to method" % key
            )
        params[key] = val
    del params['kwargs']

    collection_formats = {}

    path_params = {}

    query_params = []

    header_params = {}

    form_params = []
    local_var_files = {}

    body_params = body
    # HTTP header `Accept`
    accept = params.get('accept', 'application/json')
    if not (isinstance(accept, list) or isinstance(accept, tuple)):
        accept = [accept]
    header_params['Accept'] = self.api_client.select_header_accept(accept)

    # HTTP header `Content-Type`
    header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
        [params.get('content_type', 'application/json')])  # noqa: E501

    # Authentication setting
    auth_settings = ['api_key', 'x-xdr-auth-id']  # noqa: E501

    return self.api_client.call_api(
        path, method,
        path_params,
        query_params,
        header_params,
        body=body_params,
        post_params=form_params,
        files=local_var_files,
        response_type=params.get('response_type', 'str'),
        auth_settings=auth_settings,
        async_req=params.get('async_req'),
        _return_http_data_only=params.get('_return_http_data_only'),
        _preload_content=params.get('_preload_content', True),
        _request_timeout=params.get('_request_timeout'),
        collection_formats=collection_formats)


def get_layouts_url_for_demisto_version(api_client, params):
    """
    This function returns the correct url to upload a layout according to Demisto version that is being used.
    :param api_client: "ApiClient" instance.
    :param params: dict containing information about the uploaded layout.

    :return: the correct url as string.
    """
    url = '/v2/layouts/import'
    server_details, status_code, response_headers = api_client.call_api('/about', 'GET', header_params={
        'Accept': 'application/json'},
        auth_settings=['api_key'],
        response_type=str,
        _preload_content=params.get(
        '_preload_content', True))
    if 200 == status_code:
        server_details = json.loads(server_details.replace('\'', '"'))
        if LooseVersion(server_details.get('demistoVersion')) >= LooseVersion('6.0.0'):
            url = '/layouts/import'
    return url
